"""Test cases for the structural class definitions."""

from typing import Self

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.structural_class import (
    AbstractConcreteStructuralClassCalculator,
    ConcreteStructuralClassBase,
)
from blueprints.materials.concrete import ConcreteMaterial
from tests.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.test_exposure_classes import (
    DummyCarbonation,
    DummyChemical,
    DummyChloride,
    DummyChlorideSeawater,
    DummyExposureClasses,
    DummyFreezeThaw,
)


class MockConcreteStructuralClassCalculator(AbstractConcreteStructuralClassCalculator):
    """Mock class for the AbstractConcreteStructuralClassCalculator."""

    source_document = "Mock"
    DEFAULT_STRUCTURAL_CLASS = 1
    DEFAULT_EXPLANATION = "Default structural class (S1)"

    def __init__(
        self,
        exposure_classes: DummyExposureClasses,
        design_working_life: float,
        concrete_material: ConcreteMaterial,
        plate_geometry: bool,
        quality_control: bool,
    ) -> None:
        """Initializer of the MockConcreteStructuralClassCalculator class."""
        super().__init__(exposure_classes, design_working_life, concrete_material, plate_geometry, quality_control)

    def _structural_class_delta_concrete_grade(self) -> None:
        pass

    def _structural_class_delta_design_working_life(self) -> None:
        pass

    def _structural_class_delta_quality_control(self) -> None:
        if self.quality_control:
            self.update_structural_class(1, "Quality control")

    def _structural_class_delta_plate_geometry(self) -> None:
        pass


class MockConcreteStructuralClass(ConcreteStructuralClassBase):
    """Mock class for the ConcreteStructuralClassBase."""

    def __new__(
        cls,
        *args,
        **kwargs,
    ) -> Self:
        """Constructor of the StructuralClass class.

        Parameters
        ----------
            value (int): The value of the structural class

        Returns
        -------
            Self (ConcreteStructuralClass): The instance of the ConcreteStructuralClass class
        """
        return super().__new__(cls, MockConcreteStructuralClassCalculator, *args, **kwargs)


@pytest.fixture
def structural_class() -> MockConcreteStructuralClass:
    """Fixture that returns an instance of the structural class for testing."""
    exposure_classes = DummyExposureClasses(
        dummy_carbonation=DummyCarbonation.XC1,
        dummy_chloride=DummyChloride.NA,
        dummy_chloride_seawater=DummyChlorideSeawater.NA,
        dummy_freeze_thaw=DummyFreezeThaw.NA,
        dummy_chemical=DummyChemical.XA1,
    )
    design_working_life = 50.0
    concrete_material = ConcreteMaterial()
    plate_geometry = True
    quality_control = True
    return MockConcreteStructuralClass(
        exposure_classes=exposure_classes,
        design_working_life=design_working_life,
        concrete_material=concrete_material,
        plate_geometry=plate_geometry,
        quality_control=quality_control,
    )


@pytest.fixture
def calculator() -> MockConcreteStructuralClassCalculator:
    """Fixture that returns an instance of the calculator for testing."""
    exposure_classes = DummyExposureClasses(
        dummy_carbonation=DummyCarbonation.XC1,
        dummy_chloride=DummyChloride.NA,
        dummy_chloride_seawater=DummyChlorideSeawater.NA,
        dummy_freeze_thaw=DummyFreezeThaw.NA,
        dummy_chemical=DummyChemical.XA1,
    )
    design_working_life = 50.0
    concrete_material = ConcreteMaterial()
    plate_geometry = True
    quality_control = True
    return MockConcreteStructuralClassCalculator(
        exposure_classes=exposure_classes,
        design_working_life=design_working_life,
        concrete_material=concrete_material,
        plate_geometry=plate_geometry,
        quality_control=quality_control,
    )


class TestConcreteStructuralClassBase:
    """Test class for the ConcreteStructuralClassBase."""

    def test_concrete_structural_class_base_creation(self, structural_class: MockConcreteStructuralClass) -> None:
        """Test case to check the creation of ConcreteStructuralClassBase instance."""
        assert isinstance(structural_class, ConcreteStructuralClassBase)
        assert isinstance(structural_class, int)

    def test_concrete_structural_class_base_invalid_calculator(self) -> None:
        """Test case to check the creation of ConcreteStructuralClassBase instance with invalid calculator."""
        with pytest.raises(TypeError):
            ConcreteStructuralClassBase(MockConcreteStructuralClass, 1)  # type: ignore[arg-type]

    def test_concrete_structural_class_base_explanation(self, structural_class: MockConcreteStructuralClass) -> None:
        """Test case to check the explanation property of ConcreteStructuralClassBase."""
        assert hasattr(structural_class, "explanation")
        assert isinstance(structural_class.explanation, str)

    def test_concrete_structural_class_base_explanation_setter(self, structural_class: MockConcreteStructuralClass) -> None:
        """Test case to check the explanation setter of ConcreteStructuralClassBase."""
        with pytest.raises(AttributeError):
            structural_class.explanation = "Test explanation"

    def test_concrete_structural_class_base_string_representation(self, structural_class: MockConcreteStructuralClass) -> None:
        """Test case to check the string representation of ConcreteStructuralClassBase."""
        assert isinstance(str(structural_class), str)


class TestAbstractConcreteStructuralClassCalculator:
    """Test class for the AbstractConcreteStructuralClassCalculator."""

    @pytest.mark.parametrize(
        ("attribute", "calculated"),
        [
            ("exposure_classes", False),
            ("design_working_life", False),
            ("concrete_material", False),
            ("plate_geometry", False),
            ("quality_control", False),
            ("non_affecting_attribute", True),
        ],
    )
    def test_setattr_attribute_affecting_calculation(
        self, attribute: str, calculated: bool, calculator: MockConcreteStructuralClassCalculator
    ) -> None:
        """Test case to check if the right attributes affecting calculation trigger recalculation
        and those not affecting do not trigger recalculation.
        """
        calculator.calculate_structural_class()
        assert calculator._calculated  # noqa: SLF001

        setattr(calculator, attribute, 0)
        assert calculator._calculated == calculated  # noqa: SLF001

    def test_source_document(self, calculator: MockConcreteStructuralClassCalculator) -> None:
        """Test case to check the source document."""
        assert calculator.source_document == "Mock"

    def test_structural_class(self, calculator: MockConcreteStructuralClassCalculator) -> None:
        """Test case to check the structural class if calculate_structural_class() is not yet called."""
        with pytest.raises(ValueError):
            calculator.structural_class

    def test_explanation(self, calculator: MockConcreteStructuralClassCalculator) -> None:
        """Test case to check the explanation."""
        with pytest.raises(ValueError):
            calculator.explanation

    def test_update_structural_class(self, calculator: MockConcreteStructuralClassCalculator) -> None:
        """Test case to update the structural class and explanation."""
        calculator.update_structural_class(2, "Test explanation")
        assert calculator._structural_class == 3  # noqa: SLF001
        assert calculator._explanation == "Default structural class (S1) + 2 classes (Test explanation)"  # noqa: SLF001

    def test_update_structural_class_delta_not_int(self, calculator: MockConcreteStructuralClassCalculator) -> None:
        """Test case to check if update structural class raises error if delta is not an int."""
        with pytest.raises(TypeError):
            calculator.update_structural_class(2.2, "Test explanation")  # type: ignore[arg-type]

    def test_calculate_structural_class(self, calculator: MockConcreteStructuralClassCalculator) -> None:
        """Test case to calculate the structural class."""
        calculator.calculate_structural_class()
        assert calculator.structural_class == 2
        assert calculator.explanation == "Default structural class (S1) + 1 class (Quality control)"



"""Testing formula 5.9 of EN 1993-5:2007."""

import itertools

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_9 import Form5Dot9ReducedBendingMomentResistance
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot9ReducedBendingMomentResistance:
    """Validation for formula 5.9 from EN 1993-5:2007."""

    @pytest.mark.parametrize(
        ("mc_rd", "expected_value"),
        [
            (400, 99.9779),  # Mv,Rd < Mc,Rd
            (50, 50),  # Mv,Rd < Mc,Rd
        ],
    )
    def test_evaluation(self, mc_rd: float, expected_value: float) -> None:
        """Test the evaluation of the result."""
        # Example values
        beta_b = 0.8  # dimensionless
        w_pl = 500 * 1e3  # mm³
        rho = 0.25  # dimensionless
        a_v = 100  # mm²
        t_w = 10  # mm
        alpha = 45  # degrees
        f_y = 250  # MPa
        gamma_m_0 = 1.0  # dimensionless

        form = Form5Dot9ReducedBendingMomentResistance(
            beta_b=beta_b,
            w_pl=w_pl,
            rho=rho,
            a_v=a_v,
            t_w=t_w,
            alpha=alpha,
            f_y=f_y,
            gamma_m_0=gamma_m_0,
            mc_rd=mc_rd,
        )

        assert form == pytest.approx(expected_value)

    @pytest.mark.parametrize(
        ("invalid_argument", "invalid_value"),
        itertools.product(("beta_b", "w_pl", "rho", "a_v", "t_w", "alpha", "f_y", "gamma_m_0", "mc_rd"), (-1.0, 0.0)),
    )
    def test_raise_error_when_invalid_values_are_given(self, invalid_argument: str, invalid_value: float) -> None:
        """Test a zero and negative value for parameters."""
        kwargs = {
            key: (invalid_value if key == invalid_argument else 1.0)
            for key in ("beta_b", "w_pl", "rho", "a_v", "t_w", "alpha", "f_y", "gamma_m_0", "mc_rd")
        }
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot9ReducedBendingMomentResistance(**kwargs)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"M_{V,Rd} = \min \left\{\left(\beta_b \cdot W_{pl} - \frac{\rho \cdot A_v^2}{4 \cdot t_w \cdot \sin(\alpha)}\right) \cdot "
                r"\frac{f_y}{\gamma_{M0}}, M_{c,Rd}\right\} "
                r"= \min \left\{\left(0.90 \cdot 3070000.00 - \frac{0.01 \cdot 7805.00^2}{4 \cdot 12.20 \cdot "
                r"\sin(55.00)}\right) \cdot \frac{355.00}{0.81} \cdot 10^{-6}, 1500.00\right\} = 1204.27",
            ),
            ("short", r"M_{V,Rd} = 1204.27"),
            (
                "string",
                r"M_{V,Rd} = \min \left\{\left(\beta_b \cdot W_{pl} - \frac{\rho \cdot A_v^2}{4 \cdot t_w \cdot \sin(\alpha)}\right) \cdot "
                r"\frac{f_y}{\gamma_{M0}}, M_{c,Rd}\right\} "
                r"= \min \left\{\left(0.90 \cdot 3070000.00 - \frac{0.01 \cdot 7805.00^2}{4 \cdot 12.20 \cdot "
                r"\sin(55.00)}\right) \cdot \frac{355.00}{0.81} \cdot 10^{-6}, 1500.00\right\} = 1204.27",
            ),
        ],
    )
    def test_latex_output(self, representation: str, expected: str) -> None:
        """Test the latex implementation."""
        beta_b = 0.9  # dimensionless
        w_pl = 3070 * 1e3  # mm³
        rho = 0.01  # dimensionless
        a_v = 7805  # mm²
        t_w = 12.2  # mm
        alpha = 55  # degrees
        f_y = 355  # MPa
        gamma_m_0 = 0.81  # dimensionless
        mc_rd = 1500  # kNm

        form = Form5Dot9ReducedBendingMomentResistance(
            beta_b=beta_b,
            w_pl=w_pl,
            rho=rho,
            a_v=a_v,
            t_w=t_w,
            alpha=alpha,
            f_y=f_y,
            gamma_m_0=gamma_m_0,
            mc_rd=mc_rd,
        ).latex()

        actual = {
            "complete": form.complete,
            "short": form.short,
            "string": str(form),
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Formula 7.14 from EN 1992-1-1:2004: Chapter 7 - Serviceability limit state (SLS)."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form7Dot14MaximumCrackSpacing(Formula):
    r"""Class representing formula 7.14 for the calculation of crack spacing [$s_{r,max}$]."""

    label = "7.14"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        h: MM,
        x: MM,
    ) -> None:
        r"""[$s_{r,max}$] Where the spacing of the bonded reinforcement exceeds 5(c+⌀/2) (see Figure 7.2) or where
        there is no bonded reinforcement within the tension zone, an upper bound to the crack width
        may be found by assuming a maximum crack spacing with this formula [$mm$].

        EN 1992-1-1:2004 art.7.3.4(3) - Formula (7.14)

        Parameters
        ----------
        h : MM
            [$h$] Depth of the neutral axis [$mm$].
        x : MM
            [$x$] Depth of the concrete tension surface [$mm$].
        """
        super().__init__()
        self.h = h
        self.x = x

    @staticmethod
    def _evaluate(
        h: MM,
        x: MM,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(h=h, x=x)

        return 1.3 * (h - x)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.14."""
        _equation: str = r"1.3 \cdot (h - x)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"h": f"{self.h:.{n}f}",
                r"x": f"{self.x:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"s_{r,max}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="mm",
        )



"""Testing formula 5.13 of EN 1993-5:2007."""

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_13 import Form5Dot13SimplifiedBucklingCheck
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError

COMPLETE_LATEX_REPRESENTATION_OK = (
    r"CHECK \to \frac{N_{Ed}}{\chi \cdot (A \cdot f_{y} / \gamma_{M0}) \cdot \left( \frac{\gamma_{M0}}{\gamma_{M1}} \right)} + "
    r"1.15 \cdot \frac{M_{Ed}}{M_{c,Rd} \cdot \left( \frac{\gamma_{M0}}{\gamma_{M1}} \right)} \leq 1.0"
    r" \to \frac{100.00}{0.90 \cdot (2000.00 / 10^6 \cdot 355.0 \cdot 10^3 / 1.0) \cdot \left( \frac{1.0}{1.0} \right)} + "
    r"1.15 \cdot \frac{50.00}{100.00 \cdot \left( \frac{1.0}{1.0} \right)} \leq 1.0 \to OK"
)

COMPLETE_LATEX_REPRESENTATION_NOT_OK = (
    r"CHECK \to \frac{N_{Ed}}{\chi \cdot (A \cdot f_{y} / \gamma_{M0}) \cdot \left( \frac{\gamma_{M0}}{\gamma_{M1}} \right)} + "
    r"1.15 \cdot \frac{M_{Ed}}{M_{c,Rd} \cdot \left( \frac{\gamma_{M0}}{\gamma_{M1}} \right)} \leq 1.0"
    r" \to \frac{10000.00}{0.90 \cdot (2000.00 / 10^6 \cdot 355.0 \cdot 10^3 / 1.0) \cdot \left( \frac{1.0}{1.0} \right)} + "
    r"1.15 \cdot \frac{50.00}{100.00 \cdot \left( \frac{1.0}{1.0} \right)} \leq 1.0 \to \text{Not OK}"
)


class TestForm5Dot13SimplifiedBucklingCheck:
    """Validation for formula 5.13 from EN 1993-5:2007."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed = 100.0  # kN
        m_ed = 50.0  # kNm
        a = 2000.0  # mm^2
        f_y = 355.0  # MPa
        gamma_m0 = 1.0  # dimensionless
        gamma_m1 = 1.0  # dimensionless
        chi = 0.9  # dimensionless
        m_c_rd = 110.0  # kNm

        # Object to test
        formula = Form5Dot13SimplifiedBucklingCheck(n_ed=n_ed, m_ed=m_ed, a=a, f_y=f_y, gamma_m0=gamma_m0, gamma_m1=gamma_m1, chi=chi, m_c_rd=m_c_rd)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("n_ed", "m_ed", "a", "f_y", "gamma_m0", "gamma_m1", "chi", "m_c_rd"),
        [
            (-100.0, 50.0, 2000.0, 355.0, 1.0, 1.0, 0.9, 100.0),  # n_ed is negative
            (100.0, -50.0, 2000.0, 355.0, 1.0, 1.0, 0.9, 100.0),  # m_ed is negative
            (100.0, 50.0, -2000.0, 355.0, 1.0, 1.0, 0.9, 100.0),  # a is negative
            (100.0, 50.0, 0, 355.0, 1.0, 1.0, 0.9, 100.0),  # a is zero
            (100.0, 50.0, 2000.0, -355.0, 1.0, 1.0, 0.9, 100.0),  # f_y is negative
            (100.0, 50.0, 2000.0, 0, 1.0, 1.0, 0.9, 100.0),  # f_y is zero
            (100.0, 50.0, 2000.0, 355.0, -1.0, 1.0, 0.9, 100.0),  # gamma_m0 is negative
            (100.0, 50.0, 2000.0, 355.0, 0.0, 1.0, 0.9, 100.0),  # gamma_m0 is zero
            (100.0, 50.0, 2000.0, 355.0, 1.0, -1.0, 0.9, 100.0),  # gamma_m1 is negative
            (100.0, 50.0, 2000.0, 355.0, 1.0, 0.0, 0.9, 100.0),  # gamma_m1 is zero
            (100.0, 50.0, 2000.0, 355.0, 1.0, 1.0, -0.9, 100.0),  # chi is negative
            (100.0, 50.0, 2000.0, 355.0, 1.0, 1.0, 0, 100.0),  # chi is zero
            (100.0, 50.0, 2000.0, 355.0, 1.0, 1.0, 0.9, -100.0),  # m_c_rd is negative
            (100.0, 50.0, 2000.0, 355.0, 1.0, 1.0, 0.9, 0),  # m_c_rd is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, n_ed: float, m_ed: float, a: float, f_y: float, gamma_m0: float, gamma_m1: float, chi: float, m_c_rd: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot13SimplifiedBucklingCheck(n_ed=n_ed, m_ed=m_ed, a=a, f_y=f_y, gamma_m0=gamma_m0, gamma_m1=gamma_m1, chi=chi, m_c_rd=m_c_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                COMPLETE_LATEX_REPRESENTATION_OK,
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed = 100.0  # kN
        m_ed = 50.0  # kNm
        a = 2000.0  # mm^2
        f_y = 355.0  # MPa
        gamma_m0 = 1.0  # dimensionless
        gamma_m1 = 1.0  # dimensionless
        chi = 0.9  # dimensionless
        m_c_rd = 100.0  # kNm

        # Object to test
        latex = Form5Dot13SimplifiedBucklingCheck(
            n_ed=n_ed, m_ed=m_ed, a=a, f_y=f_y, gamma_m0=gamma_m0, gamma_m1=gamma_m1, chi=chi, m_c_rd=m_c_rd
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                COMPLETE_LATEX_REPRESENTATION_NOT_OK,
            ),
            ("short", r"CHECK \to \text{Not OK}"),
        ],
    )
    def test_latex_not_ok(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula "Not OK" case."""
        # Example values
        n_ed = 10000.0  # kN
        m_ed = 50.0  # kNm
        a = 2000.0  # mm^2
        f_y = 355.0  # MPa
        gamma_m0 = 1.0  # dimensionless
        gamma_m1 = 1.0  # dimensionless
        chi = 0.9  # dimensionless
        m_c_rd = 100.0  # kNm

        # Object to test
        latex = Form5Dot13SimplifiedBucklingCheck(
            n_ed=n_ed, m_ed=m_ed, a=a, f_y=f_y, gamma_m0=gamma_m0, gamma_m1=gamma_m1, chi=chi, m_c_rd=m_c_rd
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.31 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_31 import (
    Form6Dot31CheckTorsionShearResistanceRectangular,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot31CheckTorsionShearResistanceRectangular:
    """Validation for formula 6.31 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        t_ed = 100.0  # Nmm
        v_ed = 200.0  # N
        t_rd_c = 150.0  # Nmm
        v_rd_c = 250.0  # N

        # Object to test
        formula = Form6Dot31CheckTorsionShearResistanceRectangular(t_ed=t_ed, v_ed=v_ed, t_rd_c=t_rd_c, v_rd_c=v_rd_c)

        # Expected result, manually calculated
        expected_result = False

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("t_ed", "v_ed", "t_rd_c", "v_rd_c"),
        [
            (-100.0, 200.0, 150.0, 250.0),  # t_ed is negative
            (100.0, -200.0, 150.0, 250.0),  # v_ed is negative
            (100.0, 200.0, -150.0, 250.0),  # t_rd_c is negative
            (100.0, 200.0, 150.0, -250.0),  # v_rd_c is negative
            (100.0, 200.0, 0.0, 250.0),  # t_rd_c is zero
            (100.0, 200.0, 150.0, 0.0),  # v_rd_c is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, t_ed: float, v_ed: float, t_rd_c: float, v_rd_c: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot31CheckTorsionShearResistanceRectangular(t_ed=t_ed, v_ed=v_ed, t_rd_c=t_rd_c, v_rd_c=v_rd_c)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \frac{T_{Ed}}{T_{Rd,c}} + \frac{V_{Ed}}{V_{Rd,c}} \leq 1 \right) \to "
                r"\left( \frac{100.000}{150.000} + \frac{200.000}{250.000} \leq 1 \right) \to \text{Not OK}",
            ),
            ("short", r"CHECK \to \text{Not OK}"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        t_ed = 100.0  # Nmm
        v_ed = 200.0  # N
        t_rd_c = 150.0  # Nmm
        v_rd_c = 250.0  # N

        # Object to test
        latex = Form6Dot31CheckTorsionShearResistanceRectangular(t_ed=t_ed, v_ed=v_ed, t_rd_c=t_rd_c, v_rd_c=v_rd_c).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.77 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_77 import Form6Dot77FatigueVerification
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot77FatigueVerification:
    """Validation for formula 6.77 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        sigma_c_max = 15.0
        sigma_c_min = 5.0
        f_cd_fat = 20.0
        f_ck = 30.0

        # Object to test
        formula = Form6Dot77FatigueVerification(
            sigma_c_max=sigma_c_max,
            sigma_c_min=sigma_c_min,
            f_cd_fat=f_cd_fat,
            f_ck=f_ck,
        )

        # Expected result, manually calculated
        expected_result = False

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("sigma_c_max", "sigma_c_min", "f_cd_fat", "f_ck"),
        [
            (-15.0, 5.0, 20.0, 30.0),  # sigma_c_max is negative
            (15.0, -5.0, 20.0, 30.0),  # sigma_c_min is negative
            (15.0, 5.0, -20.0, 30.0),  # f_cd_fat is negative
            (15.0, 5.0, 20.0, -30.0),  # f_ck is negative
            (15.0, 5.0, 0.0, 30.0),  # f_cd_fat is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, sigma_c_max: float, sigma_c_min: float, f_cd_fat: float, f_ck: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot77FatigueVerification(
                sigma_c_max=sigma_c_max,
                sigma_c_min=sigma_c_min,
                f_cd_fat=f_cd_fat,
                f_ck=f_ck,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \frac{\sigma_{c,max}}{f_{cd,fat}} \leq \min\left(0.5 + 0.45 \cdot \frac{\sigma_{c,min}}{f_{cd,fat}}, "
                r"\begin{cases} 0.9 & \text{if } f_{ck} \leq 50 \\ 0.8 & \text{if } f_{ck} > 50 \end{cases}\right) \to "
                r"\frac{15.000}{20.000} \leq \min\left(0.5 + 0.45 \cdot \frac{5.000}{20.000}, \begin{cases} 0.9 & "
                r"\text{if } 30.000 \leq 50 \\ 0.8 & \text{if } 30.000 > 50 \end{cases}\right) \to \text{Not OK}",
            ),
            ("short", r"CHECK \to \text{Not OK}"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        sigma_c_max = 15.0
        sigma_c_min = 5.0
        f_cd_fat = 20.0
        f_ck = 30.0

        # Object to test
        latex = Form6Dot77FatigueVerification(
            sigma_c_max=sigma_c_max,
            sigma_c_min=sigma_c_min,
            f_cd_fat=f_cd_fat,
            f_ck=f_ck,
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.22 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DEG, DIMENSIONLESS, MPA
from blueprints.validations import raise_if_negative


class Form6Dot22CheckCrushingCompressionStruts(Formula):
    r"""Class representing formula 6.22 for checking the crushing of the compression struts in the flange."""

    label = "6.22"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        v_ed: MPA,
        nu: DIMENSIONLESS,
        f_cd: MPA,
        theta_f: DEG,
    ) -> None:
        r"""Check the crushing of the compression struts in the flange.

        EN 1992-1-1:2004 art.6.2.4(4) - Formula (6.22)

        Parameters
        ----------
        v_ed : MPA
            [$v_{Ed}$] Design shear force [$MPa$].
        nu : DIMENSIONLESS
            [$\nu$] Strength reduction factor [$-$].
        f_cd : MPA
            [$f_{cd}$] Design value of concrete compressive strength [$MPa$].
        theta_f : DEG
            [$\theta_{f}$] Angle of the compression struts [$-$].
        """
        super().__init__()
        self.v_ed = v_ed
        self.nu = nu
        self.f_cd = f_cd
        self.theta_f = theta_f

    @staticmethod
    def _evaluate(
        v_ed: MPA,
        nu: DIMENSIONLESS,
        f_cd: MPA,
        theta_f: DEG,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(v_ed=v_ed, f_cd=f_cd, nu=nu, theta_f=theta_f)

        return v_ed <= nu * f_cd * np.sin(np.radians(theta_f)) * np.cos(np.radians(theta_f))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.22."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"v_{Ed} \leq \nu \cdot f_{cd} \cdot \sin(\theta_{f}) \cdot \cos(\theta_{f})",
            numeric_equation=rf"{self.v_ed:.{n}f} \leq {self.nu:.{n}f} \cdot {self.f_cd:.{n}f} \cdot \sin({self.theta_f:.{n}f}) "
            rf"\cdot \cos({self.theta_f:.{n}f})",
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 5.3b of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_1 import Form5Dot1Imperfections
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_3b import Form5Dot3bTransverseForceBracedMembers
from blueprints.validations import NegativeValueError


class TestForm5Dot3bTransverseForceBracedMembers:
    """Validation for formula 5.3b from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        theta_i = 0.003  # -
        n_axial_force = 5  # kN

        # Object to test
        form_5_3b = Form5Dot3bTransverseForceBracedMembers(theta_i=theta_i, n_axial_force=n_axial_force)

        # Expected result, manually calculated
        manually_calculated_result = 0.030  # kN

        assert form_5_3b == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("theta_i", "n_axial_force"),
        [
            (-0.003, 5),
            (0.003, -5),
            (-0.003, -5),
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, theta_i: float, n_axial_force: float) -> None:
        """Test negative values."""
        with pytest.raises(NegativeValueError):
            Form5Dot3bTransverseForceBracedMembers(theta_i=theta_i, n_axial_force=n_axial_force)

    def test_integration_of_formulas_5_1_and_5_3b(self) -> None:
        """Test the integration of formulas 5.1 and 5.3b."""
        # Example values
        theta_0 = 0.005  # -
        alpha_m = 0.8  # -
        alpha_h = 0.9  # -
        n_axial_force = 5  # kN

        # Object to test
        form_5_3b = Form5Dot3bTransverseForceBracedMembers(
            theta_i=Form5Dot1Imperfections(theta_0=theta_0, alpha_m=alpha_m, alpha_h=alpha_h),
            n_axial_force=n_axial_force,
        )

        # Expected result, manually calculated
        manually_calculated_result = 0.036  # kN

        assert form_5_3b == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            ("complete", r"H_{i} = 2\theta_{i}N = 2\cdot0.003\cdot5.000 = 0.030"),
            ("short", r"H_{i} = 0.030"),
            ("string", r"H_{i} = 2\theta_{i}N = 2\cdot0.003\cdot5.000 = 0.030"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        theta_i = 0.003  # -
        n_axial_force = 5  # kN

        # Object to test
        form_5_3b_latex = Form5Dot3bTransverseForceBracedMembers(theta_i=theta_i, n_axial_force=n_axial_force).latex()

        actual = {
            "complete": form_5_3b_latex.complete,
            "short": form_5_3b_latex.short,
            "string": str(form_5_3b_latex),
        }

        assert expected == actual[representation], f"{representation} representation."



"""Testing formula 6.14 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_14 import Form6Dot14MCRdClass3
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot14MCRdClass3:
    """Validation for formula 6.14 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        w_el_min = 5000.0
        f_y = 355.0
        gamma_m0 = 1.0

        # Object to test
        formula = Form6Dot14MCRdClass3(w_el_min=w_el_min, f_y=f_y, gamma_m0=gamma_m0)

        # Expected result, manually calculated
        manually_calculated_result = 1775000.0  # Nmm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("w_el_min", "f_y", "gamma_m0"),
        [
            (-5000.0, 355.0, 1.0),  # w_el_min is negative
            (5000.0, -355.0, 1.0),  # f_y is negative
            (5000.0, 355.0, -1.0),  # gamma_m0 is negative
            (5000.0, 355.0, 0.0),  # gamma_m0 is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, w_el_min: float, f_y: float, gamma_m0: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot14MCRdClass3(w_el_min=w_el_min, f_y=f_y, gamma_m0=gamma_m0)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"M_{c,Rd} = \frac{W_{el,min} \cdot f_y}{\gamma_{M0}} = "
                r"\frac{5000.000 \cdot 355.000}{1.000} = 1775000.000 \ Nmm",
            ),
            ("short", r"M_{c,Rd} = 1775000.000 \ Nmm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        w_el_min = 5000.0
        f_y = 355.0
        gamma_m0 = 1.0

        # Object to test
        latex = Form6Dot14MCRdClass3(w_el_min=w_el_min, f_y=f_y, gamma_m0=gamma_m0).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 7.17 from EN 1992-1-1:2004: Chapter 7 - Serviceability limit state (SLS)."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM2, N
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form7Dot1MultiplicationFactorLimitSlenderness(Formula):
    r"""Class representing formula 7.17 for the calculation of the stress multiplication factor for the limit span to depth ratio."""

    label = "7.17"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_yk: N,
        a_s_req: MM2,
        a_s_prov: MM2,
    ) -> None:
        r"""[$\frac{310}{\sigma_s}$] Calculation of the multiplication factor [$-$].

        EN 1992-1-1:2004 art.7.4.2(2) - Formula (7.17)

        Parameters
        ----------
        f_yk : MPa
            [$f_{yk}$] Characteristic yield strength of the steel [$MPa$].
        a_s_req : MM2
            [$A_{s,req}$] Area of steel required at this section for ultimate limit state [$mm^2$].
        a_s_prov : MM2
            [$A_{s,prov}$] Area of steel provided at this section [$mm^2$].
        """
        super().__init__()
        self.f_yk = f_yk
        self.a_s_req = a_s_req
        self.a_s_prov = a_s_prov

    @staticmethod
    def _evaluate(
        f_yk: N,
        a_s_req: MM2,
        a_s_prov: MM2,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(f_yk=f_yk, a_s_req=a_s_req, a_s_prov=a_s_prov)

        return 500 / (f_yk * a_s_req / a_s_prov)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.17."""
        _equation: str = r"\frac{500}{f_{yk} \cdot \frac{A_{s,req}}{A_{s,prov}}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"f_{yk}": f"{self.f_yk:.{n}f}",
                r"A_{s,req}": f"{self.a_s_req:.{n}f}",
                r"A_{s,prov}": f"{self.a_s_prov:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"\frac{310}{\sigma_s}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 8.3 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_3 import (
    Form8Dot3RequiredAnchorageLength,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm8Dot3RequiredAnchorageLength:
    """Validation for formula 8.3 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        diameter = 12  # mm
        sigma_sd = 435  # MPA
        f_bd = 2.9  # MPA
        form_8_3 = Form8Dot3RequiredAnchorageLength(diameter=diameter, sigma_sd=sigma_sd, f_bd=f_bd)

        manually_calculated_result = 450  # mm

        assert form_8_3 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_negative_diameter(self) -> None:
        """Test that an error is raised when diameter is negative."""
        # example values
        diameter = -1  # mm
        sigma_sd = 435  # MPA
        f_bd = 2.9  # MPA

        with pytest.raises(NegativeValueError):
            Form8Dot3RequiredAnchorageLength(diameter=diameter, sigma_sd=sigma_sd, f_bd=f_bd)

    def test_raise_error_negative_sigma_sd(self) -> None:
        """Test that an error is raised when sigma_sd is negative."""
        # example values
        diameter = 12  # mm
        sigma_sd = -1  # MPA
        f_bd = 2.9  # MPA

        with pytest.raises(NegativeValueError):
            Form8Dot3RequiredAnchorageLength(diameter=diameter, sigma_sd=sigma_sd, f_bd=f_bd)

    def test_raise_error_negative_f_bd(self) -> None:
        """Test that an error is raised when f_bd is negative."""
        # example values
        diameter = 12  # mm
        sigma_sd = 435  # MPA
        f_bd = -1  # MPA

        with pytest.raises(LessOrEqualToZeroError):
            Form8Dot3RequiredAnchorageLength(diameter=diameter, sigma_sd=sigma_sd, f_bd=f_bd)

    def test_raise_error__zero_f_bd(self) -> None:
        """Test that an error is raised when f_bd is zero."""
        # example values
        diameter = 12  # mm
        sigma_sd = 435  # MPA
        f_bd = 0  # MPA

        with pytest.raises(LessOrEqualToZeroError):
            Form8Dot3RequiredAnchorageLength(diameter=diameter, sigma_sd=sigma_sd, f_bd=f_bd)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            (
                "complete",
                r"l_{b,rqd} = \frac{Ø}{4} \cdot \frac{\sigma_{sd}}{f_{bd}} = \frac{12}{4} \cdot \frac{435}{2.9} = 450.000",
            ),
            ("short", "l_{b,rqd} = 450.000"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the LaTeX representation."""
        diameter = 12  # MPa
        f_bd = 2.9  # MPa
        sigma_sd = 435  # MPa

        latex = Form8Dot3RequiredAnchorageLength(diameter=diameter, f_bd=f_bd, sigma_sd=sigma_sd).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Formula 6.43 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot43BetaRectangular(Formula):
    r"""Class representing formula 6.43 for the calculation of [$\beta$] for rectangular columns."""

    label = "6.43"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        ey: MM,
        ez: MM,
        by: MM,
        bz: MM,
    ) -> None:
        r"""[$\beta$] Calculation of [$\beta$].

        EN 1992-1-1:2004 art.6.4.3(3) - Formula (6.43)

        Parameters
        ----------
        ey : MM
            [$e_y$] Eccentricity along y-axis [$mm$].
        ez : MM
            [$e_z$] Eccentricity along z-axis [$mm$].
        by : MM
            [$b_y$] Dimension of the control perimeter along y-axis [$mm$].
        bz : MM
            [$b_z$] Dimension of the control perimeter along z-axis [$mm$].
        """
        super().__init__()
        self.ey = ey
        self.ez = ez
        self.by = by
        self.bz = bz

    @staticmethod
    def _evaluate(
        ey: MM,
        ez: MM,
        by: MM,
        bz: MM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(ey=ey, ez=ez)
        raise_if_less_or_equal_to_zero(by=by, bz=bz)

        return 1 + 1.8 * np.sqrt((ey / bz) ** 2 + (ez / by) ** 2)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.43."""
        _equation: str = r"1 + 1.8 \cdot \sqrt{\left(\frac{e_y}{b_z}\right)^2 + \left(\frac{e_z}{b_y}\right)^2}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"e_y": f"{self.ey:.{n}f}",
                r"e_z": f"{self.ez:.{n}f}",
                r"b_y": f"{self.by:.{n}f}",
                r"b_z": f"{self.bz:.{n}f}",
            },
            True,
        )
        return LatexFormula(
            return_symbol=r"\beta",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )



"""Formula 6.64 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MM2
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot64BondFactor(Formula):
    r"""Class representing formula 6.64 for the calculation of [$\eta$]."""

    label = "6.64"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_s: MM2,
        a_p: MM2,
        xi: DIMENSIONLESS,
        d_s: MM,
        d_p: MM,
    ) -> None:
        r"""[$\eta$] Calculation of [$\eta$].

        EN 1992-1-1:2004 art.6.8.2(2) - Formula (6.64)

        Parameters
        ----------
        a_s : MM2
            [$A_s$] Area of reinforcing steel [$mm^2$].
        a_p : MM2
            [$A_P$] Area of prestressing tendon or tendons [$mm^2$].
        xi : DIMENSIONLESS
            [$\xi$] ratio of bond strength between bonded tendons and ribbed steel in concrete. The value is subject to the
            relevant European Technical Approval. In the absence of this the values given in Table 6.2 may be used. [-].
        d_s : MM
            [$⌀_s$] Largest diameter of reinforcement [$mm$].
        d_p : MM
            [$⌀_P$] Diameter or equivalent diameter of prestressing steel [$mm$].
            $⌀_P = 1.6 \cdot \sqrt{A_P}$ for bundles
            $⌀_P = 1.75 \cdot ⌀_wire}$ for single 7 wire strands where $⌀_wire$ is the wire diameter
            $⌀_P = 1.20 \cdot ⌀_wire}$ for single 3 wire strands where $⌀_wire$ is the wire diameter
        """
        super().__init__()
        self.a_s = a_s
        self.a_p = a_p
        self.xi = xi
        self.d_s = d_s
        self.d_p = d_p

    @staticmethod
    def _evaluate(
        a_s: MM2,
        a_p: MM2,
        xi: DIMENSIONLESS,
        d_s: MM,
        d_p: MM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_s=a_s, a_p=a_p, xi=xi, d_s=d_s)
        raise_if_less_or_equal_to_zero(d_p=d_p)

        return (a_s + a_p) / (a_s + a_p * np.sqrt(xi * d_s / d_p))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.64."""
        _equation: str = r"\frac{A_s + A_P}{A_s + A_P \cdot \sqrt{\xi \cdot ⌀_s / ⌀_P}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A_s": f"{self.a_s:.{n}f}",
                r"A_P": f"{self.a_p:.{n}f}",
                r"\xi": f"{self.xi:.{n}f}",
                r"⌀_s": f"{self.d_s:.{n}f}",
                r"⌀_P": f"{self.d_p:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"\eta",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 5.39 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_39 import Form5Dot39SimplifiedCriterionBiaxialBending
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot39SimplifiedCriterionBiaxialBending:
    """Validation for formula 5.39 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_edz = 10.0  # KNM
        m_rdz = 20.0  # KNM
        m_edy = 15.0  # KNM
        m_rdy = 25.0  # KNM
        a = 0.8  # DIMENSIONLESS

        # Object to test
        formula = Form5Dot39SimplifiedCriterionBiaxialBending(m_edz=m_edz, m_rdz=m_rdz, m_edy=m_edy, m_rdy=m_rdy, a=a)

        # Expected result, manually calculated
        expected_result = False

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("m_edz", "m_rdz", "m_edy", "m_rdy", "a"),
        [
            (-10.0, 20.0, 15.0, 25.0, 0.8),  # m_edz is negative
            (10.0, -20.0, 15.0, 25.0, 0.8),  # m_rdz is negative
            (10.0, 20.0, -15.0, 25.0, 0.8),  # m_edy is negative
            (10.0, 20.0, 15.0, -25.0, 0.8),  # m_rdy is negative
            (10.0, 20.0, 15.0, 25.0, -0.8),  # a is negative
            (10.0, 0.0, 15.0, 25.0, 0.8),  # m_rdz is zero
            (10.0, 20.0, 15.0, 0.0, 0.8),  # m_rdy is zero
            (10.0, 20.0, 15.0, 25.0, 0.0),  # a is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, m_edz: float, m_rdz: float, m_edy: float, m_rdy: float, a: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot39SimplifiedCriterionBiaxialBending(m_edz=m_edz, m_rdz=m_rdz, m_edy=m_edy, m_rdy=m_rdy, a=a)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \frac{M_{Edz}}{M_{Rdz}} \right)^{a} + \left( \frac{M_{Edy}}{M_{Rdy}} \right)^{a} \leq 1 \to "
                r"\left( \frac{10.000}{20.000} \right)^{0.800} + \left( \frac{15.000}{25.000} \right)^{0.800} \leq 1 \to \text{Not OK}",
            ),
            ("short", r"CHECK \to \text{Not OK}"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        m_edz = 10.0  # KNM
        m_rdz = 20.0  # KNM
        m_edy = 15.0  # KNM
        m_rdy = 25.0  # KNM
        a = 0.8  # DIMENSIONLESS

        # Object to test
        latex = Form5Dot39SimplifiedCriterionBiaxialBending(m_edz=m_edz, m_rdz=m_rdz, m_edy=m_edy, m_rdy=m_rdy, a=a).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 2.2 from EN 1993-1-1:2005: Chapter 2: Basis of design."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_2_basic_of_design.formula_2_2 import Form2Dot2CharacteristicValueResistance
from blueprints.validations import NegativeValueError


class TestForm2Dot2CharacteristicValueResistance:
    """Validation for formula 2.2 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        r_d = 100  # kN
        gamma_mi = 1.1  # [-]

        form_2_2 = Form2Dot2CharacteristicValueResistance(r_d=r_d, gamma_mi=gamma_mi)
        # manually calculated result
        manually_calculated_result = 110

        assert form_2_2 == pytest.approx(expected=manually_calculated_result, rel=1e-9)

    def test_raise_error_if_negative_r_d(self) -> None:
        """Test that a NegativeValueError is raised when a negative value is passed for r_d."""
        r_d = -1
        gamma_mi = 1.1
        with pytest.raises(NegativeValueError):
            Form2Dot2CharacteristicValueResistance(r_d=r_d, gamma_mi=gamma_mi)

    def test_raise_error_if_negative_gamma_mi(self) -> None:
        """Test that a NegativeValueError is raised when a negative value is passed for gamma_mi."""
        r_d = 100
        gamma_mi = -1.1
        with pytest.raises(NegativeValueError):
            Form2Dot2CharacteristicValueResistance(r_d=r_d, gamma_mi=gamma_mi)

    def test_latex_complete(self) -> None:
        """Tests the latex representation of the formula."""
        r_d = 100
        gamma_mi = 1.1
        form_2_2 = Form2Dot2CharacteristicValueResistance(r_d=r_d, gamma_mi=gamma_mi)
        assert form_2_2.latex().complete == r"R_{k} = R_d \cdot \gamma_{Mi} = 100.00 \cdot 1.10 = 110.00"



"""Math helpers for Blueprints."""

import numpy as np

from blueprints.type_alias import DEG, DIMENSIONLESS
from blueprints.validations import raise_if_greater_than_90, raise_if_less_or_equal_to_zero, raise_if_negative


def cot(x: DEG) -> DIMENSIONLESS:
    """Calculate the cotangent of an angle in degrees.

    Parameters
    ----------
    x : DEG
        Angle in degrees.

    Returns
    -------
    DIMENSIONLESS
        Cotangent of the angle.
    """
    raise_if_less_or_equal_to_zero(x=x)
    raise_if_greater_than_90(x=x)
    return 1 / np.tan(np.deg2rad(x))


def sec(x: DEG) -> DIMENSIONLESS:
    """Calculate the secant of an angle in degrees.

    Parameters
    ----------
    x : DEG
        Angle in degrees.

    Returns
    -------
    DIMENSIONLESS
        Secant of the angle.
    """
    raise_if_negative(x=x)
    raise_if_greater_than_90(x=x)
    return 1 / np.cos(np.deg2rad(x))


def csc(x: DEG) -> DIMENSIONLESS:
    """Calculate the cosecant of an angle in degrees.

    Parameters
    ----------
    x : DEG
        Angle in degrees.

    Returns
    -------
    DIMENSIONLESS
        Cosecant of the angle.
    """
    raise_if_less_or_equal_to_zero(x=x)
    raise_if_greater_than_90(x=x)
    return 1 / np.sin(np.deg2rad(x))



"""Formula 5.22 from EN 1993-5:2007: Chapter 5 - Ultimate Limit States."""

from blueprints.codes.eurocode.en_1993_5_2007 import EN_1993_5_2007
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import KN, KNM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot22ReducedMomentResistanceClass3(Formula):
    r"""Class representing formula 5.22 for the calculation of [$M_N,Rd$] for Class 3 cross-sections."""

    label = "5.22"
    source_document = EN_1993_5_2007

    def __init__(
        self,
        m_c_rd: KNM,
        n_ed: KN,
        n_pl_rd: KN,
    ) -> None:
        r"""[$M_N,Rd$] Calculation of the reduced design moment resistance allowing for the axial force [$kNm$].

        EN 1993-5:2007 art. 5.2.3 (11) - Formula (5.22)

        Parameters
        ----------
        m_c_rd : KNM
            [$M_c,Rd$] Design moment resistance of the cross-section [$kNm$].
        n_ed : KN
            [$N_Ed$] Design value of the axial force [$kN$].
        n_pl_rd : KN
            [$N_pl,Rd$] Design plastic resistance of the cross-section [$kN$].
        """
        super().__init__()
        self.m_c_rd = m_c_rd
        self.n_ed = n_ed
        self.n_pl_rd = n_pl_rd

    @staticmethod
    def _evaluate(
        m_c_rd: KNM,
        n_ed: KN,
        n_pl_rd: KN,
    ) -> KNM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(m_c_rd=m_c_rd, n_ed=n_ed)
        raise_if_less_or_equal_to_zero(n_pl_rd=n_pl_rd)

        return m_c_rd * (1 - n_ed / n_pl_rd)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.22."""
        _equation: str = r"M_{c,Rd} \cdot \left(1 - \frac{N_{Ed}}{N_{pl,Rd}}\right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"M_{c,Rd}": f"{self.m_c_rd:.{n}f}",
                r"N_{Ed}": f"{self.n_ed:.{n}f}",
                r"N_{pl,Rd}": f"{self.n_pl_rd:.{n}f}",
            },
            False,
        )

        return LatexFormula(
            return_symbol=r"M_{N,Rd}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="kNm",
        )



"""Formula 6.54 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot54ControlPerimeter(Formula):
    r"""Class representing formula 6.54 for the calculation of the control perimeter at which shear reinforcement is not required."""

    label = "6.54"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        beta: DIMENSIONLESS,
        v_ed: N,
        v_rd_c: MPA,
        d: MM,
    ) -> None:
        r"""[u_out,ef] Calculation of the control perimeter at which shear reinforcement is not required.

        EN 1992-1-1:2004 art.6.5.4(4) - Formula (6.54)

        Parameters
        ----------
        beta : DIMENSIONLESS
            [$\beta$] Factor as per 6.4.3 (3), (4) and (5) [$-$].
        v_ed : N
            [$V_{Ed}$] Design shear force [$N$].
        v_rd_c : MPA
            [$v_{Rd,c}$] Design shear strength of concrete [$MPa$].
        d : MM
            [$d$] Effective depth of the slab [$mm$].
        """
        super().__init__()
        self.beta = beta
        self.v_ed = v_ed
        self.v_rd_c = v_rd_c
        self.d = d

    @staticmethod
    def _evaluate(
        beta: DIMENSIONLESS,
        v_ed: N,
        v_rd_c: MPA,
        d: MM,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(beta=beta, v_ed=v_ed)
        raise_if_less_or_equal_to_zero(v_rd_c=v_rd_c, d=d)

        return beta * v_ed / (v_rd_c * d)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.54."""
        _equation: str = r"\frac{\beta \cdot V_{Ed}}{v_{Rd,c} \cdot d}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\beta": f"{self.beta:.{n}f}",
                r"V_{Ed}": f"{self.v_ed:.{n}f}",
                r"v_{Rd,c}": f"{self.v_rd_c:.{n}f}",
                r" d": f" {self.d:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"u_{out,ef}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="mm",
        )



"""Testing formula 7.7n of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_7n import Form7Dot7nMaxBarDiameterTension
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm7Dot7nMaxBarDiameterTension:
    """Validation for formula 7.7n from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        diam_s_star = 16.0
        f_ct_eff = 2.5
        h_cr = 200.0
        h = 500.0
        d = 450.0

        # Object to test
        formula = Form7Dot7nMaxBarDiameterTension(
            diam_s_star=diam_s_star,
            f_ct_eff=f_ct_eff,
            h_cr=h_cr,
            h=h,
            d=d,
        )

        # Expected result, manually calculated
        manually_calculated_result = 6.89655172413793  # mm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("diam_s_star", "f_ct_eff", "h_cr", "h", "d"),
        [
            (-16.0, 2.5, 200.0, 500.0, 450.0),  # diam_s_star is negative
            (16.0, -2.5, 200.0, 500.0, 450.0),  # f_ct_eff is negative
            (16.0, 2.5, -200.0, 500.0, 450.0),  # h_cr is negative
            (16.0, 2.5, 200.0, -500.0, 450.0),  # h is negative
            (16.0, 2.5, 200.0, 500.0, -450.0),  # d is negative
            (16.0, 2.5, 200.0, 500.0, 500.0),  # d is equal to h
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, diam_s_star: float, f_ct_eff: float, h_cr: float, h: float, d: float) -> None:
        """Test invalid values."""
        with pytest.raises((LessOrEqualToZeroError, NegativeValueError)):
            Form7Dot7nMaxBarDiameterTension(
                diam_s_star=diam_s_star,
                f_ct_eff=f_ct_eff,
                h_cr=h_cr,
                h=h,
                d=d,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"⌀_s = ⌀^*_s \cdot \left(\frac{f_{ct,eff}}{2.9}\right) \cdot \left(\frac{h_{cr}}{8 \cdot ( h - d)}\right) = "
                r"16.000 \cdot \left(\frac{2.500}{2.9}\right) \cdot \left(\frac{200.000}{8 \cdot ( 500.000 - 450.000)}\right) "
                r"= 6.897 \ mm",
            ),
            ("short", r"⌀_s = 6.897 \ mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        diam_s_star = 16.0
        f_ct_eff = 2.5
        h_cr = 200.0
        h = 500.0
        d = 450.0

        # Object to test
        latex = Form7Dot7nMaxBarDiameterTension(
            diam_s_star=diam_s_star,
            f_ct_eff=f_ct_eff,
            h_cr=h_cr,
            h=h,
            d=d,
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 7.3 from EN 1995-1-1:2004."""

from contextlib import AbstractContextManager
from contextlib import nullcontext as does_not_raise

import pytest

from blueprints.codes.eurocode.en_1995_1_1_2004.chapter_7_serviceability_limit_states.formula_7_3 import Form7Dot3RatioDeflectionPointLoadUC
from blueprints.validations import LessOrEqualToZeroError


class TestForm7Dot3RatioDeflectionPointLoadUCt:
    """Validation for formula 7.3 from EN 1995-1-1:2004."""

    @pytest.mark.parametrize(
        ("w", "f", "alpha", "expected_result"),
        [
            (1, 1, 1, 1),
            (0.002, 1, 0.2, 0.01),
            (0.8, 1, 0.4, 2),
        ],
        ids=["UC=1", "UC<1", "UC>1"],
    )
    def test_evaluation(self, w: float, f: float, alpha: float, expected_result: float) -> None:
        """Test the evaluation of the result."""
        form_7_3 = Form7Dot3RatioDeflectionPointLoadUC(w=w, f=f, alpha=alpha)
        assert form_7_3 == pytest.approx(expected=expected_result, abs=1e-4)

    @pytest.mark.parametrize(
        ("w", "f", "alpha", "expectation"),
        [
            (1, 1, 1, does_not_raise()),
            (-1, 1, 2, pytest.raises(LessOrEqualToZeroError)),
            (1, 0, 1, pytest.raises(LessOrEqualToZeroError)),
            (1, 0, -2, pytest.raises(LessOrEqualToZeroError)),
        ],
        ids=[
            "passes",
            "w<0",
            "F=0",
            "alpha<0",
        ],
    )
    def test_raise_error_incorrect_args(self, w: float, f: float, alpha: float, expectation: AbstractContextManager) -> None:
        """Test if errors are raised."""
        with expectation:
            assert Form7Dot3RatioDeflectionPointLoadUC(w=w, f=f, alpha=alpha) is not None

    @pytest.mark.parametrize(
        ("w", "f", "alpha", "rep_short", "rep_long"),
        [
            (
                0.0020,  # DIMENSIONLESS
                1,  # HZ
                0.0200,  # DIMENSIONLESS
                r"UC = 0.100",
                r"UC = \frac{w/F}{\alpha} = \frac{0.002/1.000}{0.020} = 0.100",
            ),
            (
                1,  # DIMENSIONLESS
                1,  # HZ
                1,  # DIMENSIONLESS
                r"UC = 1.000",
                r"UC = \frac{w/F}{\alpha} = \frac{1.000/1.000}{1.000} = 1.000",
            ),
        ],
        ids=["decimal result", "integer result"],
    )
    def test_latex(self, w: float, f: float, alpha: float, rep_short: str, rep_long: str) -> None:
        """Test the latex representation of the formula."""
        form_7_3_latex = Form7Dot3RatioDeflectionPointLoadUC(w=w, f=f, alpha=alpha).latex()
        assert form_7_3_latex.complete == rep_long
        assert form_7_3_latex.short == rep_short



"""Tests reinforcement configurations."""

import pytest
from shapely import LineString

from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial
from blueprints.structural_sections.concrete.reinforced_concrete_sections.reinforcement_configurations import (
    ReinforcementByDistance,
    ReinforcementByQuantity,
)


class TestReinforcementByDistance:
    """Tests for the reinforcement by distance configuration."""

    def test_n_rebars_per_meter(self, reinforcement_by_distance: ReinforcementByDistance) -> None:
        """Test the number of rebars per meter."""
        n_rebars_per_meter = reinforcement_by_distance.n_rebars_per_meter
        assert n_rebars_per_meter == 10

    def test_area(self, reinforcement_by_distance: ReinforcementByDistance) -> None:
        """Test the area of the reinforcement."""
        area = reinforcement_by_distance.area
        assert area == pytest.approx(expected=1130.9733, rel=1e-4)

    def test__repr__(self, reinforcement_by_distance: ReinforcementByDistance) -> None:
        """Test the representation of the reinforcement."""
        representation = repr(reinforcement_by_distance)
        assert representation == "ReinforcementByDistance|⌀12-100|1131 mm²/m"

    def test_wrong_ctc(self) -> None:
        """Test the wrong center-to-center distance."""
        with pytest.raises(ValueError):
            ReinforcementByDistance(
                diameter=12,
                center_to_center=0,
                material=ReinforcementSteelMaterial(),
            )

    def test_to_rebars(self, reinforcement_by_distance: ReinforcementByDistance) -> None:
        """Test the conversion to rebars."""
        line = LineString([(0, 0), (1000, 0)])
        rebars = reinforcement_by_distance.to_rebars(line=line)
        assert len(rebars) == 10
        assert all(rebar.diameter == 12 for rebar in rebars)
        assert all(rebar.material == ReinforcementSteelMaterial() for rebar in rebars)

    def test_error_when_using_linearring(self, reinforcement_by_distance: ReinforcementByDistance) -> None:
        """Test the error when using a Linearring."""
        linearring = LineString([(0, 0), (1000, 0), (1000, 1000), (0, 1000), (0, 0)])
        with pytest.raises(ValueError):
            reinforcement_by_distance.to_rebars(line=linearring)


class TestReinforcementByQuantity:
    """Tests for the reinforcement by quantity configuration."""

    def test_area(self, reinforcement_by_quantity: ReinforcementByQuantity) -> None:
        """Test the area of the reinforcement."""
        area = reinforcement_by_quantity.area
        assert area == pytest.approx(expected=1130.9733, rel=1e-4)

    @pytest.mark.parametrize("diameter", [0, -1])
    def test_wrong_diameter(self, diameter: int) -> None:
        """Test the wrong diameter."""
        with pytest.raises(ValueError):
            ReinforcementByQuantity(
                diameter=diameter,
                material=ReinforcementSteelMaterial(),
                n=10,
            )

    @pytest.mark.parametrize(("wrong_n", "expected_error"), [(0, ValueError), (-1, ValueError), (1.5, TypeError)])
    def test_wrong_n_float(self, wrong_n: int, expected_error: type[BaseException]) -> None:
        """Test the wrong number of rebars."""
        with pytest.raises(expected_error):
            ReinforcementByQuantity(
                diameter=12,
                material=ReinforcementSteelMaterial(),
                n=wrong_n,
            )

    def test_to_rebars(self, reinforcement_by_quantity: ReinforcementByQuantity) -> None:
        """Test the conversion to rebars."""
        line = LineString([(0, 0), (1000, 0)])
        rebars = reinforcement_by_quantity.to_rebars(line=line)
        assert len(rebars) == 10
        assert all(rebar.diameter == 12 for rebar in rebars)
        assert all(rebar.material == ReinforcementSteelMaterial() for rebar in rebars)

    def test__repr__(self, reinforcement_by_quantity: ReinforcementByQuantity) -> None:
        """Test the representation of the reinforcement."""
        representation = repr(reinforcement_by_quantity)
        assert representation == "ReinforcementByQuantity|10⌀12|1131 mm²"



"""Formula 2.2 from NEN 9997-1+C2:2017: Chapter 2: Basis of geotechnical design."""

from blueprints.codes.eurocode.nen_9997_1_c2_2017 import NEN_9997_1_C2_2017
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_fraction
from blueprints.type_alias import DIMENSIONLESS
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form2Dot2DesignValueGeotechnicalParameter(Formula):
    """Class representing formula 2.2 for the calculation of the design value [$X_d$] of geotechnical parameter [$X$]."""

    label = "2.2"
    source_document = NEN_9997_1_C2_2017

    def __init__(self, x_k: DIMENSIONLESS, gamma_m: DIMENSIONLESS) -> None:
        r"""[$X_d$] Design value of geotechnical parameter [$X$].

        NEN 9997-1+C2:2017 art.2.4.6.2(1) - Formula (2.2)

        Parameters
        ----------
        x_k : DIMENSIONLESS
            [$X_{k}$] Characteristic value of geotechnical parameter [$X$].
        gamma_m : DIMENSIONLESS
            [$\gamma_M$] material partial factor [$-$].
        """
        super().__init__()
        self.x_k = x_k
        self.gamma_m = gamma_m

    @staticmethod
    def _evaluate(
        x_k: DIMENSIONLESS,
        gamma_m: DIMENSIONLESS,
    ) -> float:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(gamma_m=gamma_m)
        return x_k / gamma_m

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 2.2."""
        numerator = f"{self.x_k:.{n}f}"
        denominator = f"{self.gamma_m:.{n}f}"
        return LatexFormula(
            return_symbol="X_d",
            result=f"{self:.{n}f}",
            equation=r"\frac{X_{k}}{\gamma_M}",
            numeric_equation=f"{latex_fraction(numerator=numerator, denominator=denominator)}",
            comparison_operator_label="=",
        )



"""Formula 6.23 from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import NMM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot23CheckTorsionalMoment(Formula):
    r"""Class representing formula 6.23 for checking torsional moment."""

    label = "6.23"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        t_ed: NMM,
        t_rd: NMM,
    ) -> None:
        r"""Check the torsional moment at each cross-section.

        EN 1993-1-1:2005 art.6.2.7(1) - Formula (6.23)

        Parameters
        ----------
        t_ed : NMM
            [$T_{Ed}$] Design value of the torsional moment [$Nmm$].
        t_rd : NMM
            [$T_{Rd}$] Design torsional resistance of the cross section [$Nmm$].
        """
        super().__init__()
        self.t_ed = t_ed
        self.t_rd = t_rd

    @staticmethod
    def _evaluate(
        t_ed: NMM,
        t_rd: NMM,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(t_rd=t_rd)
        raise_if_negative(t_ed=t_ed)

        return t_ed / t_rd <= 1.0

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.23."""
        _equation: str = r"\left( \frac{T_{Ed}}{T_{Rd}} \leq 1.0 \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                "T_{Ed}": f"{self.t_ed:.{n}f}",
                "T_{Rd}": f"{self.t_rd:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 12.2 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_12_plain_and_lightly_reinforced_concrete_structures.formula_12_2 import (
    Form12Dot2PlainConcreteBendingResistance,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm12Dot2PlainConcreteBendingResistance:
    """Validation for formula 12.2 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        eta_f_cd_pl = 30.0  # MPa
        b = 300.0  # mm
        h_w = 500.0  # mm
        e = 100.0  # mm

        # Object to test
        form_12_2 = Form12Dot2PlainConcreteBendingResistance(eta_f_cd_pl=eta_f_cd_pl, b=b, h_w=h_w, e=e)

        # Expected result, manually calculated
        manually_calculated_result = 2700000.0  # N

        assert round(form_12_2, 3) == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("eta_f_cd_pl", "b", "h_w", "e"),
        [
            (-30.0, 300.0, 500.0, 100.0),
            (30.0, -300.0, 500.0, 100.0),
            (30.0, 300.0, 500.0, -100.0),
        ],
    )
    def test_raise_error_when_negative_values_are_given(
        self,
        eta_f_cd_pl: float,
        b: float,
        h_w: float,
        e: float,
    ) -> None:
        """Test negative values for eta_f_cd_pl, b, h_w, and e."""
        with pytest.raises(NegativeValueError):
            Form12Dot2PlainConcreteBendingResistance(eta_f_cd_pl=eta_f_cd_pl, b=b, h_w=h_w, e=e)

    @pytest.mark.parametrize(
        ("eta_f_cd_pl", "b", "h_w"),
        [
            (30.0, 300.0, 0.0),
            (30.0, 300.0, -20),
        ],
    )
    def test_raise_error_when_values_are_less_or_equal_to_zero(
        self,
        eta_f_cd_pl: float,
        b: float,
        h_w: float,
    ) -> None:
        """Test values less or equal to zero for eta_f_cd_pl, b, and h_w."""
        with pytest.raises(LessOrEqualToZeroError):
            Form12Dot2PlainConcreteBendingResistance(eta_f_cd_pl=eta_f_cd_pl, b=b, h_w=h_w, e=100.0)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"N_{Rd} = \eta f_{cd,pl} \cdot b \cdot h_w \cdot \left(1 - \frac{2e}{h_w}\right) = 30.000 \cdot "
                r"300.000 \cdot 500.000 \cdot \left(1 - \frac{2 \cdot 100.000}{500.000}\right) = 2700000.000",
            ),
            ("short", r"N_{Rd} = 2700000.000"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        eta_f_cd_pl = 30.0  # MPa
        b = 300.0  # mm
        h_w = 500.0  # mm
        e = 100.0  # mm

        # Object to test
        form_12_2_latex = Form12Dot2PlainConcreteBendingResistance(
            eta_f_cd_pl=eta_f_cd_pl,
            b=b,
            h_w=h_w,
            e=e,
        ).latex()

        actual = {
            "complete": form_12_2_latex.complete,
            "short": form_12_2_latex.short,
            "string": str(form_12_2_latex),
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Tests for constants for the calculation of nominal concrete cover according to EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.nominal_cover_constants import (
    AbrasionClass,
    CastingSurface,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover.constants import NominalConcreteCoverConstants
from blueprints.type_alias import MM


class TestNominalConcreteCoverConstants2004:
    """Tests for the constants class for the calculation of nominal concrete cover according to EN 1992-1-1:2004."""

    def test_instantiation(self) -> None:
        """Test that the class can be instantiated."""
        constants = NominalConcreteCoverConstants()
        assert constants.COVER_INCREASE_FOR_UNEVEN_SURFACE == 5
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
            AbrasionClass.XM1: 5,
            AbrasionClass.XM2: 10,
            AbrasionClass.XM3: 15,
        }
        assert constants.DEFAULT_DELTA_C_DEV == 10

    def test_instantiation_with_custom_cover_increase_uneven_surfaces(self) -> None:
        """Test that the class cannot be instantiated with custom value for the cover increase for uneven surfaces."""
        with pytest.raises(TypeError, match=".* got an unexpected keyword argument .*"):
            _ = NominalConcreteCoverConstants(  # type: ignore[call-arg]
                COVER_INCREASE_FOR_UNEVEN_SURFACE=10,
            )

    def test_instantiation_with_custom_cover_increase_abraison_class(self) -> None:
        """Test that the class can be instantiated with custom value for the cover increase for abrasion class."""
        custom_constants = NominalConcreteCoverConstants(  # type: ignore[call-arg]
            COVER_INCREASE_FOR_ABRASION_CLASS={
                AbrasionClass.NA: 0,
            },
        )
        assert custom_constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
        }

    def test_instantiation_with_custom_delta_c_dev(self) -> None:
        """Test that the class can be instantiated with custom value for the default delta c_dev."""
        custom_constants = NominalConcreteCoverConstants(  # type: ignore[call-arg]
            DEFAULT_DELTA_C_DEV=13,
        )
        assert custom_constants.DEFAULT_DELTA_C_DEV == 13

    def test_instantiation_with_default_delta_c_dev(self) -> None:
        """Test that the class can be instantiated with default value for the default delta c_dev."""
        custom_constants = NominalConcreteCoverConstants()
        assert custom_constants.DEFAULT_DELTA_C_DEV == 10  # Default value

    def test_post_init(self) -> None:
        """Test that the post-init method sets default values."""
        constants = NominalConcreteCoverConstants()
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
            AbrasionClass.XM1: 5,
            AbrasionClass.XM2: 10,
            AbrasionClass.XM3: 15,
        }

    @pytest.mark.parametrize(
        ("c_min_dur", "casting_surface", "expected_result"),
        [
            (10, CastingSurface.PERMANENTLY_EXPOSED, 0),
            (10, CastingSurface.FORMWORK, 0),
            (10, CastingSurface.PREPARED_GROUND, 50),
            (10, CastingSurface.DIRECTLY_AGAINST_SOIL, 85),
        ],
    )
    def test_minimum_cover_with_regard_to_casting_surface(self, c_min_dur: MM, casting_surface: CastingSurface, expected_result: MM) -> None:
        """Test the method for the calculation of the minimum cover with regard to casting surface."""
        constants = NominalConcreteCoverConstants()
        assert constants.minimum_cover_with_regard_to_casting_surface(c_min_dur, casting_surface) == expected_result

    @pytest.mark.parametrize(
        ("casting_surface", "expected_result"),
        [
            (CastingSurface.PERMANENTLY_EXPOSED, "0 (No additional requirements for Permanently exposed)"),
            (CastingSurface.FORMWORK, "0 (No additional requirements for Formwork)"),
            (CastingSurface.PREPARED_GROUND, "k1 \\ge c_{min,dur} + 40 mm for Prepared ground (including blinding)"),
            (CastingSurface.DIRECTLY_AGAINST_SOIL, "k2 \\ge c_{min,dur} + 75 mm for Directly against soil"),
        ],
    )
    def test_minimum_cover_with_regard_to_casting_surface_latex(self, casting_surface: CastingSurface, expected_result: str) -> None:
        """Test the method for the calculation of the minimum cover with regard to casting surface in LaTeX."""
        constants = NominalConcreteCoverConstants()
        assert constants.minimum_cover_with_regard_to_casting_surface_latex(casting_surface) == expected_result



"""Testing formula 5.6 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_6 import Form5Dot6TransverseForceEffectRoofDiaphragm
from blueprints.validations import NegativeValueError


class TestForm5Dot6TransverseForceEffectRoofDiaphragm:
    """Validation for formula 5.6 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        theta_i = 0.003  # -
        n_a = 5  # kN

        # Object to test
        form_5_6 = Form5Dot6TransverseForceEffectRoofDiaphragm(theta_i=theta_i, n_a=n_a)

        # Expected result, manually calculated
        manually_calculated_result = 0.015  # kN

        assert form_5_6 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_theta_i_is_given(self) -> None:
        """Test a negative value for theta_i."""
        # Example values
        theta_i = -0.003
        n_a = 5

        with pytest.raises(NegativeValueError):
            Form5Dot6TransverseForceEffectRoofDiaphragm(theta_i=theta_i, n_a=n_a)

    def test_raise_error_when_negative_n_a_is_given(self) -> None:
        """Test a negative value for n_a."""
        # Example values
        theta_i = 0.003
        n_a = -5

        with pytest.raises(NegativeValueError):
            Form5Dot6TransverseForceEffectRoofDiaphragm(theta_i=theta_i, n_a=n_a)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"H_{i} = Θ_{i} \cdot N_{a} = 0.003 \cdot 5.000 = 0.015"),
            ("short", "H_{i} = 0.015"),
            ("string", r"H_{i} = Θ_{i} \cdot N_{a} = 0.003 \cdot 5.000 = 0.015"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        theta_i = 0.003  # -
        n_a = 5  # kN

        # Object to test
        form_5_6_latex = Form5Dot6TransverseForceEffectRoofDiaphragm(
            theta_i=theta_i,
            n_a=n_a,
        ).latex()

        actual = {
            "complete": form_5_6_latex.complete,
            "short": form_5_6_latex.short,
            "string": str(form_5_6_latex),
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Testing formula 9.4 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_9_detailling_and_specific_rules.formula_9_4 import Form9Dot4ShearReinforcementRatio
from blueprints.validations import GreaterThan90Error, NegativeValueError


class TestForm9Dot4ShearReinforcementRatio:
    """Validation for formula 9.4 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        a_sw = 100  # mm²
        s = 200  # mm
        b_w = 150  # mm
        alpha = 85  # deg
        form_9_4 = Form9Dot4ShearReinforcementRatio(
            a_sw=a_sw,
            s=s,
            b_w=b_w,
            alpha=alpha,
        )

        # Expected result, manually calculated
        manually_calculated_result = 0.003346066

        assert form_9_4 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_a_sw_is_given(self) -> None:
        """Test if error is raised when a_sw is negative."""
        a_sw = -100  # mm²
        s = 200  # mm
        b_w = 150  # mm
        alpha = 85  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot4ShearReinforcementRatio(
                a_sw=a_sw,
                s=s,
                b_w=b_w,
                alpha=alpha,
            )

    def test_raise_error_when_negative_s_is_given(self) -> None:
        """Test if error is raised when s is negative."""
        a_sw = 100  # mm²
        s = -200  # mm
        b_w = 150  # mm
        alpha = 85  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot4ShearReinforcementRatio(
                a_sw=a_sw,
                s=s,
                b_w=b_w,
                alpha=alpha,
            )

    def test_raise_error_when_negative_b_w_is_given(self) -> None:
        """Test if error is raised when b_w is negative."""
        a_sw = 100  # mm²
        s = 200  # mm
        b_w = -150  # mm
        alpha = 85  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot4ShearReinforcementRatio(
                a_sw=a_sw,
                s=s,
                b_w=b_w,
                alpha=alpha,
            )

    def test_raise_error_when_negative_alpha_is_given(self) -> None:
        """Test if error is raised when alpha is negative."""
        a_sw = 100  # mm²
        s = 200  # mm
        b_w = 150  # mm
        alpha = -85  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot4ShearReinforcementRatio(
                a_sw=a_sw,
                s=s,
                b_w=b_w,
                alpha=alpha,
            )

    def test_raise_error_when_alpha_is_greater_90(self) -> None:
        """Test if error is raised when alpha is negative."""
        a_sw = 100  # mm²
        s = 200  # mm
        b_w = 150  # mm
        alpha = 110  # deg

        with pytest.raises(GreaterThan90Error):
            Form9Dot4ShearReinforcementRatio(
                a_sw=a_sw,
                s=s,
                b_w=b_w,
                alpha=alpha,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\rho_w = \frac{A_{sw}}{s \cdot b_w \cdot sin(\alpha)} = \frac{100.00}{200.00 \cdot 150.00 \cdot sin(85.00)} = 0.003346",
            ),
            ("short", r"\rho_w = 0.003346"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        a_sw = 100  # mm²
        s = 200  # mm
        b_w = 150  # mm
        alpha = 85  # deg

        # Object to test
        form_9_4_latex = Form9Dot4ShearReinforcementRatio(
            a_sw=a_sw,
            s=s,
            b_w=b_w,
            alpha=alpha,
        ).latex()

        actual = {"complete": form_9_4_latex.complete, "short": form_9_4_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 8.13 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_13 import (
    Form8Dot13AdditionalShearReinforcement,
)
from blueprints.validations import NegativeValueError


class TestForm8Dot13AdditionalShearReinforcement:
    """Validation for formula 8.13 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        a_s = 100  # mm²
        n_2 = 2  # -
        form_8_12 = Form8Dot13AdditionalShearReinforcement(a_s=a_s, n_2=n_2)

        manually_calculated_result = 50  # mm²

        assert form_8_12 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_negative_a_s(self) -> None:
        """Test that an error is raised when a_s is negative."""
        # example values
        a_s = -100  # mm²
        n_2 = 2  # -

        with pytest.raises(NegativeValueError):
            Form8Dot13AdditionalShearReinforcement(a_s=a_s, n_2=n_2)

    def test_raise_error_negative_n_2(self) -> None:
        """Test that an error is raised when n_2 is negative."""
        # example values
        a_s = 100  # mm²
        n_2 = -2  # -

        with pytest.raises(NegativeValueError):
            Form8Dot13AdditionalShearReinforcement(a_s=a_s, n_2=n_2)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            ("complete", r"A_{sv} = 0.25 \cdot A_s \cdot n_2 = 0.25 \cdot 100.00 \cdot 2.00 = 50.00"),
            ("short", r"A_{sv} = 50.00"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # example values
        a_s = 100  # mm²
        n_2 = 2  # -

        # Object to test
        form_8_13_latex = Form8Dot13AdditionalShearReinforcement(a_s=a_s, n_2=n_2).latex()

        actual = {"complete": form_8_13_latex.complete, "short": form_8_13_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 3.2 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_3_materials.formula_3_2 import Form3Dot2CoefficientDependentOfConcreteAge


class TestForm3Dot2CoefficientDependentOfConcreteAge:
    """Validation for formula 3.2 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        s = 0.25  # -
        t = 10  # days
        form_3_2 = Form3Dot2CoefficientDependentOfConcreteAge(s=s, t=t)

        # Expected result, manually calculated
        manually_calculated_result = 0.84507490

        assert form_3_2 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_t_is_given(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        s = 0.25  # -
        t = -10  # days

        with pytest.raises(ValueError):
            Form3Dot2CoefficientDependentOfConcreteAge(s=s, t=t)

    def test_raise_error_when_negative_s_is_given(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        s = -0.9  # -
        t = 10  # days

        with pytest.raises(ValueError):
            Form3Dot2CoefficientDependentOfConcreteAge(s=s, t=t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\beta_{cc}(t) = \exp \left( s \cdot \left( 1 - \left( \frac{28}{t} \right) ^{1/2} \right) \right) = "
                r"\exp \left( 0.250 \cdot \left( 1 - \left( \frac{28}{10.000} \right) ^{1/2} \right) \right) = 0.845",
            ),
            ("short", r"\beta_{cc}(t) = 0.845"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        s = 0.25  # -
        t = 10  # days

        # Object to test
        form_3_2_latex = Form3Dot2CoefficientDependentOfConcreteAge(s=s, t=t).latex()

        actual = {"complete": form_3_2_latex.complete, "short": form_3_2_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 5.34 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_34 import Form5Dot34Curvature
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot34Curvature:
    """Validation for formula 5.34 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        k_r = 0.8
        k_phi = 1.2
        f_yd = 500.0
        e_s = 200000.0
        d = 300.0

        # Object to test
        formula = Form5Dot34Curvature(k_r=k_r, k_phi=k_phi, f_yd=f_yd, e_s=e_s, d=d)

        # Expected result, manually calculated
        manually_calculated_result = 0.0000177777777777777777  # 1/mm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-9)

    @pytest.mark.parametrize(
        ("k_r", "k_phi", "f_yd", "e_s", "d"),
        [
            (-0.8, 1.2, 500.0, 200000.0, 300.0),  # k_r is negative
            (0.8, -1.2, 500.0, 200000.0, 300.0),  # k_phi is negative
            (0.8, 1.2, -500.0, 200000.0, 300.0),  # f_yd is negative
            (0.8, 1.2, 500.0, -200000.0, 300.0),  # e_s is negative
            (0.8, 1.2, 500.0, 200000.0, -300.0),  # d is negative
            (0.8, 1.2, 0.0, 200000.0, 300.0),  # f_yd is zero
            (0.8, 1.2, 500.0, 0.0, 300.0),  # e_s is zero
            (0.8, 1.2, 500.0, 200000.0, 0.0),  # d is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, k_r: float, k_phi: float, f_yd: float, e_s: float, d: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot34Curvature(k_r=k_r, k_phi=k_phi, f_yd=f_yd, e_s=e_s, d=d)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\frac{1}{r} = K_r \cdot K_\phi \cdot \frac{f_{yd}}{E_s \cdot 0.45 \cdot d} "
                r"= 0.800 \cdot 1.200 \cdot \frac{500.000}{200000.000 \cdot 0.45 \cdot 300.000} = 0.000018 \ 1/mm",
            ),
            ("short", r"\frac{1}{r} = 0.000018 \ 1/mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        k_r = 0.8
        k_phi = 1.2
        f_yd = 500.0
        e_s = 200000.0
        d = 300.0

        # Object to test
        latex = Form5Dot34Curvature(k_r=k_r, k_phi=k_phi, f_yd=f_yd, e_s=e_s, d=d).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 8.21 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_21 import (
    Form8Dot21AnchorageLength,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm8Dot21DispersionLength:
    """Validation for formula 8.21 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        l_pt2 = 100.0  # mm
        alpha_2 = 0.8
        diameter = 12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 2.5  # MPa
        form_8_21 = Form8Dot21AnchorageLength(
            l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd
        )
        # manually calculated result
        manually_calculated_result = 484.0  # mm

        assert form_8_21 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_if_l_pt2_is_negative(self) -> None:
        """Test that a NegativeValueError is raised when l_pt2 is negative."""
        l_pt2 = -100.0  # mm
        alpha_2 = 0.8
        diameter = 12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot21AnchorageLength(l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd)

    def test_raise_error_if_alpha_2_is_negative(self) -> None:
        """Test that a NegativeValueError is raised when alpha_2 is negative."""
        l_pt2 = 100.0  # mm
        alpha_2 = -0.8
        diameter = 12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot21AnchorageLength(l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd)

    def test_raise_error_if_diameter_is_negative(self) -> None:
        """Test that a NegativeValueError is raised when diameter is negative."""
        l_pt2 = 100.0  # mm
        alpha_2 = 0.8
        diameter = -12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot21AnchorageLength(l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd)

    def test_raise_error_if_sigma_pd_is_negative(self) -> None:
        """Test that a NegativeValueError is raised when sigma_pd is negative."""
        l_pt2 = 100.0  # mm
        alpha_2 = 0.8
        diameter = 12.0  # mm
        sigma_pd = -1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot21AnchorageLength(l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd)

    def test_raise_error_if_sigma_pminf_is_negative(self) -> None:
        """Test that a NegativeValueError is raised when sigma_pminf is negative."""
        l_pt2 = 100.0  # mm
        alpha_2 = 0.8
        diameter = 12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = -900.0  # MPa
        f_bpd = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot21AnchorageLength(l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd)

    def test_raise_error_if_f_bpd_is_zero_or_negative(self) -> None:
        """Test that a LessOrEqualToZeroError is raised when f_bpd is zero or negative."""
        l_pt2 = 100.0  # mm
        alpha_2 = 0.8
        diameter = 12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 0.0  # MPa
        with pytest.raises(LessOrEqualToZeroError):
            Form8Dot21AnchorageLength(l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"l_{bpd} = l_{pt2} + \alpha_{2} \cdot Ø \cdot \frac{\sigma_{pd} - \sigma_{pm\infty}}{f_{bpd}} = "
                r"100.000 + 0.800 \cdot 12.000 \cdot \frac{1000.000 - 900.000}{2.500} = 484.000",
            ),
            ("short", r"l_{bpd} = 484.000"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # example values
        l_pt2 = 100.0  # mm
        alpha_2 = 0.8
        diameter = 12.0  # mm
        sigma_pd = 1000.0  # MPa
        sigma_pminf = 900.0  # MPa
        f_bpd = 2.5  # MPa
        form_8_21 = Form8Dot21AnchorageLength(
            l_pt2=l_pt2, alpha_2=alpha_2, diameter=diameter, sigma_pd=sigma_pd, sigma_pminf=sigma_pminf, f_bpd=f_bpd
        )

        # Object to test
        form_8_21_latex = form_8_21.latex()

        actual = {"complete": form_8_21_latex.complete, "short": form_8_21_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 3.28 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_3_materials.formula_3_28 import Form3Dot28RatioLossOfPreStressClass1


class TestForm3Dot28RatioLossOfPreStressClass1:
    """Validation for formula 3.28 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        rho_1000 = 35.2  # %
        mu = 0.28  # -
        t = 7.4  # hours

        form_3_28 = Form3Dot28RatioLossOfPreStressClass1(rho_1000=rho_1000, mu=mu, t=t)

        # Expected result, manually calculated
        manually_calculated_result = 8.754937 * 10**-4

        assert form_3_28 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_rho_1000_is_given(self) -> None:
        """Test a negative value."""
        # Example values
        rho_1000 = -35.2  # %
        mu = 0.28  # -
        t = 7.4  # hours

        with pytest.raises(ValueError):
            Form3Dot28RatioLossOfPreStressClass1(rho_1000=rho_1000, mu=mu, t=t)

    def test_raise_error_when_negative_t_is_given(self) -> None:
        """Test a negative value."""
        # Example values
        rho_1000 = 35.2  # %
        mu = 0.28  # -
        t = -7.4  # hours

        with pytest.raises(ValueError):
            Form3Dot28RatioLossOfPreStressClass1(rho_1000=rho_1000, mu=mu, t=t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\frac{\Delta \sigma_{pr}}{\sigma_{pl}} = "
                r"5.39 \cdot \rho_{1000} \cdot e^{6.7 \cdot \mu} \left( \frac{t}{1000} \right)^{0.75 \cdot (1 - \mu)} \cdot 10^{-5} = "
                r"5.39 \cdot 35.200 \cdot e^{6.7 \cdot 0.280} \left( \frac{7.400}{1000} \right)^{0.75 \cdot (1 - 0.280)} \cdot 10^{-5} = 0.000875",
            ),
            ("short", r"\frac{\Delta \sigma_{pr}}{\sigma_{pl}} = 0.000875"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        rho_1000 = 35.2  # %
        mu = 0.28  # -
        t = 7.4  # hours

        # Object to test
        form_3_28_latex = Form3Dot28RatioLossOfPreStressClass1(rho_1000=rho_1000, mu=mu, t=t).latex()

        actual = {"complete": form_3_28_latex.complete, "short": form_3_28_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 5.10 of EN 1993-5:2007."""

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_10 import Form5Dot10ReductionFactorShearArea
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot10ReductionFactorShearArea:
    """Validation for formula 5.10 from EN 1993-5:2007."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        v_ed = 100  # kN
        v_pl_rd = 400  # kN

        form = Form5Dot10ReductionFactorShearArea(v_ed=v_ed, v_pl_rd=v_pl_rd)

        # Expected result, manually calculated
        expected = 0.25

        assert form == pytest.approx(expected)

    @pytest.mark.parametrize(
        ("v_ed", "v_pl_rd"),
        [
            (0, 200),  # v_ed is zero
            (-100, 200),  # v_ed is negative
            (100, 0),  # v_pl_rd is zero
            (100, -200),  # v_pl_rd is negative
        ],
    )
    def test_raise_error_when_negative_or_zero_values_are_given(self, v_ed: float, v_pl_rd: float) -> None:
        """Test a zero and negative value for parameters v_ed and v_pl_rd."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot10ReductionFactorShearArea(v_ed=v_ed, v_pl_rd=v_pl_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\rho = \left(2 \cdot \frac{V_{Ed}}{V_{pl,Rd}} - 1\right)^2 = \left(2 \cdot \frac{100}{400} - 1\right)^2 = 0.25",
            ),
            ("short", r"\rho = 0.25"),
            (
                "string",
                r"\rho = \left(2 \cdot \frac{V_{Ed}}{V_{pl,Rd}} - 1\right)^2 = \left(2 \cdot \frac{100}{400} - 1\right)^2 = 0.25",
            ),
        ],
    )
    def test_latex_output(self, representation: str, expected: str) -> None:
        """Test the latex implementation."""
        # Example values
        v_ed = 100  # kN
        v_pl_rd = 400  # kN

        form = Form5Dot10ReductionFactorShearArea(v_ed=v_ed, v_pl_rd=v_pl_rd).latex()

        actual = {
            "complete": form.complete,
            "short": form.short,
            "string": str(form),
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 5.101 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_2_2005.chapter_5_structural_analysis.formula_5_101 import (
    Form5Dot101Imperfections,
    Form5Dot101Sub1ReductionFactorLengthOrHeight,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot101Imperfections:
    """Validation for formula 5.101 from EN 1992-2:2005."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        theta_0 = 0.005
        alpha_h = 0.8

        # Object to test
        form_5_1 = Form5Dot101Imperfections(theta_0=theta_0, alpha_h=alpha_h)

        # Expected result, manually calculated
        manually_calculated_result = 0.004

        assert form_5_1 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_theta_0_is_given(self) -> None:
        """Test a negative value of Θ0."""
        # Example values
        theta_0 = -0.005
        alpha_h = 0.8

        with pytest.raises(NegativeValueError):
            Form5Dot101Imperfections(theta_0=theta_0, alpha_h=alpha_h)

    def test_raise_error_when_negative_alpha_h_is_given(self) -> None:
        """Test a negative value of αh."""
        # Example values
        theta_0 = 0.005
        alpha_h = -0.8

        with pytest.raises(NegativeValueError):
            Form5Dot101Imperfections(theta_0=theta_0, alpha_h=alpha_h)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\theta_i = \theta_0 \cdot \alpha_h = 0.005 \cdot 0.800 = 0.0040",
            ),
            ("short", r"\theta_i = 0.0040"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        theta_0 = 0.005
        alpha_h = 0.8

        # Object to test
        form_5_1_latex = Form5Dot101Imperfections(theta_0=theta_0, alpha_h=alpha_h).latex()

        actual = {
            "complete": form_5_1_latex.complete,
            "short": form_5_1_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."


class TestForm5Dot101Sub1ReductionFactorLengthOrHeight:
    """Validation for subformula of 5.101 from EN 1992-2:2005."""

    @pytest.mark.parametrize(
        ("length", "expected"),
        [
            (5.3, 0.86874448552),  # Example value
            (0.683, 1),  # Example value where the result should be capped at 1
        ],
    )
    def test_evaluation_sub1(self, length: float, expected: float) -> None:
        """Test the evaluation of the result."""
        # Object to test
        form_5_1_sub1 = Form5Dot101Sub1ReductionFactorLengthOrHeight(length)

        assert form_5_1_sub1 == pytest.approx(expected=expected, rel=1e-4)

    @pytest.mark.parametrize(
        "length",
        [
            -5.3,  # length is negative
            0.0,  # length is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given_sub1(self, length: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot101Sub1ReductionFactorLengthOrHeight(length)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\alpha_h = \min(2 / \sqrt{l}, 1) = \min( 2 / \sqrt{5.300}, 1) = 0.869",
            ),
            ("short", r"\alpha_h = 0.869"),
        ],
    )
    def test_latex_sub1(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        length = 5.3

        # Object to test
        form_5_1_sub1_latex = Form5Dot101Sub1ReductionFactorLengthOrHeight(length).latex()

        actual = {
            "complete": form_5_1_sub1_latex.complete,
            "short": form_5_1_sub1_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Module for the concrete exposure classes
according to Table 4.1 from EN 1992-1-1:2004: Chapter 4 - Durability and cover to reinforcement.
"""

from functools import total_ordering

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.exposure_classes import (
    Exposure,
    ExposureClassesBase,
)


@total_ordering
class Carbonation(Exposure):
    """Enum Class which indicates the classification of corrosion induced by carbonation."""

    NA = "Not applicable"
    XC1 = "XC1"
    XC2 = "XC2"
    XC3 = "XC3"
    XC4 = "XC4"

    @staticmethod
    def exposure_class_description() -> str:
        """Static method which returns the description of this exposure class.

        Returns
        -------
        str
            description of this exposure class
        """
        return "Corrosion induced by carbonation"

    def description_of_the_environment(self) -> str:
        """Description of the environment based on the instance.

        Returns
        -------
        str
            description of the environment based on the instance
        """
        match self:
            case Carbonation.XC1:
                return "Dry or permanently wet"
            case Carbonation.XC2:
                return "Wet, rarely dry"
            case Carbonation.XC3:
                return "Moderate humidity"
            case Carbonation.XC4:
                return "Cyclic wet and dry"
            case Carbonation.NA:
                return "Not applicable"


@total_ordering
class Chloride(Exposure):
    """Enum Class which indicates the classification of corrosion induced by chlorides other than by sea water."""

    NA = "Not applicable"
    XD1 = "XD1"
    XD2 = "XD2"
    XD3 = "XD3"

    @staticmethod
    def exposure_class_description() -> str:
        """Static method which returns the description of this exposure class.

        Returns
        -------
        str
            description of this exposure class
        """
        return "Corrosion induced by chlorides other than by sea water"

    def description_of_the_environment(self) -> str:
        """Description of the environment based on the instance.

        Returns
        -------
        str
            description of the environment based on the instance
        """
        match self:
            case Chloride.XD1:
                return "Moderate humidity"
            case Chloride.XD2:
                return "Wet, rarely dry"
            case Chloride.XD3:
                return "Cyclic wet and dry"
            case Chloride.NA:
                return "Not applicable"


@total_ordering
class ChlorideSeawater(Exposure):
    """Enum Class which indicates the classification of corrosion induced by chlorides from sea water."""

    NA = "Not applicable"
    XS1 = "XS1"
    XS2 = "XS2"
    XS3 = "XS3"

    @staticmethod
    def exposure_class_description() -> str:
        """Static method which returns the description of this exposure class.

        Returns
        -------
        str
            description of this exposure class
        """
        return "Corrosion induced by chlorides from sea water"

    def description_of_the_environment(self) -> str:
        """Description of the environment based on the instance.

        Returns
        -------
        str
            description of the environment based on the instance
        """
        match self:
            case ChlorideSeawater.XS1:
                return "Exposed to airborne salt but not in direct contact with sea water"
            case ChlorideSeawater.XS2:
                return "Permanently submerged"
            case ChlorideSeawater.XS3:
                return "Tidal, splash and spray zones"
            case ChlorideSeawater.NA:
                return "Not applicable"


@total_ordering
class FreezeThaw(Exposure):
    """Enum Class which indicates the classification of freeze/thaw attack with or without de-icing agents."""

    NA = "Not applicable"
    XF1 = "XF1"
    XF2 = "XF2"
    XF3 = "XF3"
    XF4 = "XF4"

    @staticmethod
    def exposure_class_description() -> str:
        """Static method which returns the description of this exposure class.

        Returns
        -------
        str
            description of this exposure class
        """
        return "Freeze/thaw attack with or without de-icing agents"

    def description_of_the_environment(self) -> str:
        """Description of the environment based on the instance.

        Returns
        -------
        str
            description of the environment based on the instance
        """
        match self:
            case FreezeThaw.XF1:
                return "Moderate water saturation, without de-icing agent"
            case FreezeThaw.XF2:
                return "Moderate water saturation, with de-icing agent"
            case FreezeThaw.XF3:
                return "High water saturation, without de-icing agents"
            case FreezeThaw.XF4:
                return "High water saturation with de-icing agents or sea water"
            case FreezeThaw.NA:
                return "Not applicable"


@total_ordering
class Chemical(Exposure):
    """Enum Class which indicates the classification of chemical attack."""

    NA = "Not applicable"
    XA1 = "XA1"
    XA2 = "XA2"
    XA3 = "XA3"

    @staticmethod
    def exposure_class_description() -> str:
        """Static method which returns the description of this exposure class.

        Returns
        -------
        str
            description of this exposure class
        """
        return "Chemical attack"

    def description_of_the_environment(self) -> str:
        """Description of the environment based on the instance.

        Returns
        -------
        str
            description of the environment based on the instance
        """
        match self:
            case Chemical.XA1:
                return "Slightly aggressive chemical environment according to EN 206-1, Table 2"
            case Chemical.XA2:
                return "Moderately aggressive chemical environment according to EN 206-1, Table 2"
            case Chemical.XA3:
                return "Highly aggressive chemical environment according to EN 206-1, Table 2"
            case Chemical.NA:
                return "Not applicable"


class Table4Dot1ExposureClasses(ExposureClassesBase):
    """Class representing table 4.1 from EN 1992-1-1:2004."""

    def __init__(
        self,
        carbonation: Carbonation = Carbonation.NA,
        chloride: Chloride = Chloride.NA,
        chloride_seawater: ChlorideSeawater = ChlorideSeawater.NA,
        freeze_thaw: FreezeThaw = FreezeThaw.NA,
        chemical: Chemical = Chemical.NA,
    ) -> None:
        """Implementation of table 4.1 from EN 1992-1-1:2004 par. 4.2.

        Exposure classes related to environmental conditions in accordance with EN 206-1

        Parameters
        ----------
        carbonation : Carbonation
            The carbonation exposure class. Defaults to Carbonation.NA.
        chloride : Chloride
            The chloride exposure class. Defaults to Chloride.NA.
        chloride_seawater : ChlorideSeawater
            The chloride seawater exposure class. Defaults to ChlorideSeawater.NA.
        freeze_thaw : FreezeThaw
            The freeze/thaw exposure class. Defaults to FreezeThaw.NA.
        chemical : Chemical
            The chemical exposure class. Defaults to Chemical.NA.
        """
        self.carbonation = carbonation
        self.chloride = chloride
        self.chloride_seawater = chloride_seawater
        self.freeze_thaw = freeze_thaw
        self.chemical = chemical



"""Testing formula 6.29rho of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_29rho import (
    Form6Dot29Rho,
    Form6Dot29RhoWithTorsion,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot29Rho:
    """Validation for formula 6.29rho from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_ed = 300.0
        v_pl_rd = 500.0

        # Object to test
        formula = Form6Dot29Rho(v_ed=v_ed, v_pl_rd=v_pl_rd)

        # Expected result, manually calculated
        manually_calculated_result = 0.04  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation2(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_ed = 100.0
        v_pl_rd = 500.0

        # Object to test
        formula = Form6Dot29Rho(v_ed=v_ed, v_pl_rd=v_pl_rd)

        # Expected result, manually calculated
        manually_calculated_result = 0.0  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("v_ed", "v_pl_rd"),
        [
            (-300.0, 500.0),  # v_ed is negative
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, v_ed: float, v_pl_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot29Rho(v_ed=v_ed, v_pl_rd=v_pl_rd)

    @pytest.mark.parametrize(
        ("v_ed", "v_pl_rd"),
        [
            (300.0, 0.0),  # v_pl_rd is zero
            (300.0, -500.0),  # v_pl_rd is negative
        ],
    )
    def test_raise_error_when_less_or_equal_to_zero_values_are_given(self, v_ed: float, v_pl_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot29Rho(v_ed=v_ed, v_pl_rd=v_pl_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\rho = \begin{cases} "
                r"0 & \text{if } V_{Ed} \leq 0.5 \cdot V_{pl,Rd} \\ "
                r"\left( \frac{2 \cdot V_{Ed}}{V_{pl,Rd}} - 1 \right)^2 & \text{if } V_{Ed} > 0.5 \cdot V_{pl,Rd} "
                r"\end{cases} = \begin{cases} "
                r"0 & \text{if } 300.000 \leq 0.5 \cdot 500.000 \\ "
                r"\left( \frac{2 \cdot 300.000}{500.000} - 1 \right)^2 & \text{if } 300.000 > 0.5 \cdot 500.000 "
                r"\end{cases} = 0.040 \ -",
            ),
            (
                "complete_with_units",
                r"\rho = \begin{cases} "
                r"0 & \text{if } V_{Ed} \leq 0.5 \cdot V_{pl,Rd} \\ "
                r"\left( \frac{2 \cdot V_{Ed}}{V_{pl,Rd}} - 1 \right)^2 & \text{if } V_{Ed} > 0.5 \cdot V_{pl,Rd} "
                r"\end{cases} = \begin{cases} "
                r"0 & \text{if } 300.000 \ N \leq 0.5 \cdot 500.000 \ N \\ "
                r"\left( \frac{2 \cdot 300.000 \ N}{500.000 \ N} - 1 \right)^2 & \text{if } 300.000 \ N > 0.5 \cdot 500.000 \ N "
                r"\end{cases} = 0.040 \ -",
            ),
            ("short", r"\rho = 0.040 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_ed = 300.0
        v_pl_rd = 500.0

        # Object to test
        latex = Form6Dot29Rho(v_ed=v_ed, v_pl_rd=v_pl_rd).latex()

        actual = {
            "complete": latex.complete,
            "complete_with_units": latex.complete_with_units,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestForm6Dot29RhoWithTorsion:
    """Validation for formula 6.29rho with torsion from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_ed = 600.0
        v_pl_t_rd = 1000.0

        # Object to test
        formula = Form6Dot29RhoWithTorsion(v_ed=v_ed, v_pl_t_rd=v_pl_t_rd)

        # Expected result, manually calculated
        manually_calculated_result = 0.04  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation2(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_ed = 100.0
        v_pl_t_rd = 1000.0

        # Object to test
        formula = Form6Dot29RhoWithTorsion(v_ed=v_ed, v_pl_t_rd=v_pl_t_rd)

        # Expected result, manually calculated
        manually_calculated_result = 0.00  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("v_ed", "v_pl_t_rd"),
        [
            (-600.0, 1000.0),  # v_ed is negative
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, v_ed: float, v_pl_t_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot29RhoWithTorsion(v_ed=v_ed, v_pl_t_rd=v_pl_t_rd)

    @pytest.mark.parametrize(
        ("v_ed", "v_pl_t_rd"),
        [
            (600.0, 0.0),  # v_pl_t_rd is zero
            (600.0, -1000.0),  # v_pl_t_rd is negative
        ],
    )
    def test_raise_error_when_less_or_equal_to_zero_values_are_given(self, v_ed: float, v_pl_t_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot29RhoWithTorsion(v_ed=v_ed, v_pl_t_rd=v_pl_t_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\rho = \begin{cases} "
                r"0 & \text{if } V_{Ed} \leq 0.5 \cdot V_{pl,T,Rd} \\ "
                r"\left( \frac{2 \cdot V_{Ed}}{V_{pl,T,Rd}} - 1 \right)^2 & \text{if } V_{Ed} > 0.5 \cdot V_{pl,T,Rd} "
                r"\end{cases} = \begin{cases} "
                r"0 & \text{if } 600.000 \leq 0.5 \cdot 1000.000 \\ "
                r"\left( \frac{2 \cdot 600.000}{1000.000} - 1 \right)^2 & \text{if } 600.000 > 0.5 \cdot 1000.000 "
                r"\end{cases} = 0.040 \ -",
            ),
            (
                "complete_with_units",
                r"\rho = \begin{cases} "
                r"0 & \text{if } V_{Ed} \leq 0.5 \cdot V_{pl,T,Rd} \\ "
                r"\left( \frac{2 \cdot V_{Ed}}{V_{pl,T,Rd}} - 1 \right)^2 & \text{if } V_{Ed} > 0.5 \cdot V_{pl,T,Rd} "
                r"\end{cases} = \begin{cases} "
                r"0 & \text{if } 600.000 \ N \leq 0.5 \cdot 1000.000 \ N \\ "
                r"\left( \frac{2 \cdot 600.000 \ N}{1000.000 \ N} - 1 \right)^2 & \text{if } 600.000 \ N > 0.5 \cdot 1000.000 \ N "
                r"\end{cases} = 0.040 \ -",
            ),
            ("short", r"\rho = 0.040 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_ed = 600.0
        v_pl_t_rd = 1000.0

        # Object to test
        latex = Form6Dot29RhoWithTorsion(v_ed=v_ed, v_pl_t_rd=v_pl_t_rd).latex()

        actual = {
            "complete": latex.complete,
            "complete_with_units": latex.complete_with_units,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing sub-formula for 3.4 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_3_materials.formula_3_4 import SubForm3Dot4CoefficientAgeConcreteAlpha


class TestSubForm3Dot4CoefficientAgeConcreteAlpha:
    """Validation for sub-formula 3.4 from EN 1992-1-1:2004."""

    def test_t_between_0_and_28(self) -> None:
        """Test the evaluation of the result."""
        # Example value 1
        t = 10  # days

        sub_form_3_4 = SubForm3Dot4CoefficientAgeConcreteAlpha(t=t)

        # Expected result, manually calculated
        manually_result = 1.0

        assert sub_form_3_4 == pytest.approx(expected=manually_result, rel=1e-4)

    def test_raise_error_when_t_lower_or_equal_to_0(self) -> None:
        """Test the evaluation of the result."""
        # Example value 1
        t = 0  # days

        with pytest.raises(ValueError):
            SubForm3Dot4CoefficientAgeConcreteAlpha(t=t)

    def test_t_higher_then_28(self) -> None:
        """Test the evaluation of the result."""
        # Example value 1
        t = 50  # days

        sub_form_3_4 = SubForm3Dot4CoefficientAgeConcreteAlpha(t=t)

        # Expected result, manually calculated
        manually_result = 2 / 3

        assert sub_form_3_4 == pytest.approx(expected=manually_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\alpha \rightarrow t \rightarrow 10.000 \rightarrow 1.000",
            ),
            ("short", r"\alpha \rightarrow 1.000"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        t = 10  # days

        # Object to test
        form_3_4_latex = SubForm3Dot4CoefficientAgeConcreteAlpha(t=t).latex()

        actual = {"complete": form_3_4_latex.complete, "short": form_3_4_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 7.21 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_21 import Form7Dot21CurvatureDueToShrinkage
from blueprints.validations import NegativeValueError


class TestForm7Dot21CurvatureDueToShrinkage:
    """Validation for formula 7.21 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        epsilon_cs = 0.0003  # dimensionless
        es = 200000.0  # MPa
        ec_eff = 30000.0  # MPa
        capital_s = 50000.0  # mm^3
        capital_i = 200000.0  # mm^4

        # Object to test
        formula = Form7Dot21CurvatureDueToShrinkage(epsilon_cs=epsilon_cs, es=es, ec_eff=ec_eff, capital_s=capital_s, capital_i=capital_i)

        # Expected result, manually calculated
        manually_calculated_result = 0.0005  # dimensionless

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("epsilon_cs", "es", "ec_eff", "capital_s", "capital_i"),
        [
            (-0.0003, 200000.0, 30000.0, 50000.0, 200000.0),  # epsilon_cs is negative
            (0.0003, -200000.0, 30000.0, 50000.0, 200000.0),  # es is negative
            (0.0003, 200000.0, -30000.0, 50000.0, 200000.0),  # ec_eff is negative
            (0.0003, 200000.0, 30000.0, -50000.0, 200000.0),  # capital_s is negative
            (0.0003, 200000.0, 30000.0, 50000.0, -200000.0),  # capital_i is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, epsilon_cs: float, es: float, ec_eff: float, capital_s: float, capital_i: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form7Dot21CurvatureDueToShrinkage(epsilon_cs=epsilon_cs, es=es, ec_eff=ec_eff, capital_s=capital_s, capital_i=capital_i)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\frac{1}{r_{cs}} = \epsilon_{cs} \cdot \frac{E_s}{E_{c,eff}} \cdot \frac{S}{I} = "
                r"0.0003 \cdot \frac{200000.000}{30000.000} \cdot \frac{50000.000}{200000.000} = 0.000500 \ mm^{-1}",
            ),
            ("short", r"\frac{1}{r_{cs}} = 0.000500 \ mm^{-1}"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        epsilon_cs = 0.0003  # dimensionless
        es = 200000.0  # MPa
        ec_eff = 30000.0  # MPa
        capital_s = 50000.0  # mm^3
        capital_i = 200000.0  # mm^4

        # Object to test
        latex = Form7Dot21CurvatureDueToShrinkage(epsilon_cs=epsilon_cs, es=es, ec_eff=ec_eff, capital_s=capital_s, capital_i=capital_i).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.30 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_30 import Form6Dot30DesignTorsionalResistanceMoment
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot30DesignTorsionalResistanceMoment:
    """Validation for formula 6.30 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        nu = 0.6
        alpha_cw = 1.0
        f_cd = 30.0
        a_k = 1000.0
        t_ef_i = 200.0
        theta = 45.0

        # Object to test
        formula = Form6Dot30DesignTorsionalResistanceMoment(nu=nu, alpha_cw=alpha_cw, f_cd=f_cd, a_k=a_k, t_ef_i=t_ef_i, theta=theta)

        # Expected result, manually calculated
        manually_calculated_result = 3600000.0

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("nu", "alpha_cw", "f_cd", "a_k", "t_ef_i", "theta"),
        [
            (-0.6, 1.0, 30.0, 1000.0, 200.0, 45.0),  # nu is negative
            (0.6, -1.0, 30.0, 1000.0, 200.0, 45.0),  # alpha_cw is negative
            (0.6, 1.0, -30.0, 1000.0, 200.0, 45.0),  # f_cd is negative
            (0.6, 1.0, 30.0, -1000.0, 200.0, 45.0),  # a_k is negative
            (0.6, 1.0, 30.0, 1000.0, -200.0, 45.0),  # t_ef_i is negative
            (0.6, 1.0, 30.0, 1000.0, 200.0, -45.0),  # theta is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, nu: float, alpha_cw: float, f_cd: float, a_k: float, t_ef_i: float, theta: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot30DesignTorsionalResistanceMoment(nu=nu, alpha_cw=alpha_cw, f_cd=f_cd, a_k=a_k, t_ef_i=t_ef_i, theta=theta)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"T_{Rd,max} = 2 \cdot \nu \cdot \alpha_{cw} \cdot f_{cd} \cdot A_{k} \cdot t_{ef,i} \cdot \sin(\theta) \cdot \cos(\theta) = "
                r"2 \cdot 0.600 \cdot 1.000 \cdot 30.000 \cdot 1000.000 \cdot 200.000 \cdot \sin(45.000) \cdot \cos(45.000) = 3600000.000 "
                r"\ Nmm",
            ),
            ("short", r"T_{Rd,max} = 3600000.000 \ Nmm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        nu = 0.6
        alpha_cw = 1.0
        f_cd = 30.0
        a_k = 1000.0
        t_ef_i = 200.0
        theta = 45.0

        # Object to test
        latex = Form6Dot30DesignTorsionalResistanceMoment(nu=nu, alpha_cw=alpha_cw, f_cd=f_cd, a_k=a_k, t_ef_i=t_ef_i, theta=theta).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 3.4 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DAYS, DIMENSIONLESS, MPA


class Form3Dot4DevelopmentTensileStrength(Formula):
    """Class representing formula 3.4 for an initial estimation of the tensile strength, fctm(t), after t days."""

    label = "3.4"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        beta_cc_t: DIMENSIONLESS,
        alpha: DIMENSIONLESS,
        f_ctm: MPA,
    ) -> None:
        r"""[$f_{ctm}(t)$] The initial estimation of the tensile strength after t days [MPa].

        EN 1992-1-1:2004 art.3.1.2(9) - Formula (3.4)

        Parameters
        ----------
        beta_cc_t : DIMENSIONLESS
            [$\beta_{cc}(t)$] Coefficient dependent of the age of concrete [-].
        alpha : DIMENSIONLESS
            [$\alpha$] Factor dependent of the age of concrete [-]
            alpha = 1 for t < 28 days
            alpha = 2/3 for t >= 28 days
            Use your own implementation of this value or use the SubForm3Dot4CoefficientAgeConcreteAlpha class.
        f_ctm : MPA
            [$f_{ctm}$] Tensile strength from table 3.1 [MPa].

        Returns
        -------
        None
        """
        super().__init__()
        self.beta_cc_t = beta_cc_t
        self.alpha = alpha
        self.f_ctm = f_ctm

    @staticmethod
    def _evaluate(
        beta_cc_t: DIMENSIONLESS,
        alpha: DIMENSIONLESS,
        f_ctm: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        if beta_cc_t < 0:
            raise ValueError(f"Negative beta_cc_t: {beta_cc_t}. beta_cc_t cannot be negative")
        if f_ctm < 0:
            raise ValueError(f"Negative f_ctm: {f_ctm}. f_ctm cannot be negative")
        if alpha < 0:
            raise ValueError(f"Negative alpha: {alpha}. alpha cannot be negative")
        return beta_cc_t**alpha * f_ctm

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.4."""
        return LatexFormula(
            return_symbol=r"f_{ctm}(t)",
            result=f"{self:.{n}f}",
            equation=r"(\beta_{cc}(t))^{\alpha} \cdot f_{ctm}",
            numeric_equation=rf"({self.beta_cc_t:.{n}f})^{{{self.alpha:.{n}f}}} \cdot {self.f_ctm:.{n}f}",
            comparison_operator_label=r"=",
        )


class SubForm3Dot4CoefficientAgeConcreteAlpha(Formula):
    """Class representing sub-formula for formula 3.4 for the coefficient 'α' which is dependent of the age of concrete."""

    label = "3.4"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        t: DAYS,
    ) -> None:
        r"""[$\alpha$] Factor dependent of the age of concrete [$-$].

        EN 1992-1-1:2004 art.3.1.2(9) - α

        Parameters
        ----------
        t : DAYS
            [$t$] Age of concrete in days [$days$].
        """
        super().__init__()
        self.t = t

    @staticmethod
    def _evaluate(
        t: DAYS,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        if t <= 0:
            raise ValueError(f"Invalid t: {t}. t cannot be negative or zero")
        if t < 28:
            return 1.0
        return 2 / 3

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.4sub."""
        return LatexFormula(
            return_symbol=r"\alpha",
            result=f"{self:.{n}f}",
            equation=r"t",
            numeric_equation=rf"{self.t:.{n}f}",
            comparison_operator_label=r"\rightarrow",
        )
